/*  cacherep.c  ==  cache file reporting utility  */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
**                                                                   **
**                   Copyright (C) 2011 by                           **
**     Ken Clardy, Carnegie Observatories, Pasadena California.      **
**                                                                   **
**      This software is proprietary, and may be used or copied      **
**          only with the written consent of the author.             **
**              All copies must retain this notice.                  **
**                                                                   **
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*  Accepts data file name, partial name or data file number as 
  command line argument.
  Writes to stdout a report of the content, with formatted
  translation of status line, and formatted object lines with
  positions in sexigisimal notation.  (The output may be redirected
  or piped to "more" on the command line.)
  Argument is prompted for if not supplied; multiple arguments
  are handled integelently.    */


/*  === Common includes === */
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>

#include	<sys/types.h>	// for stat
#include	<sys/stat.h>	// for stat
#include	<unistd.h>	// for access, stat
#include	<time.h>

/*  ====  GLOBAL DEFINITIONS  ====  */

const	char	PROGNAME[] = "Cache Reporting utility";

//int	dodebug;

/* mgutils seems to need text version still */

#include	MGUTILH
const	char	VERSION[] = "0.29";

#include	"catalog.h"
// Including catalog.h requires a dummy statement later


// Standard Version structure
// made from versup.c
//const	Vers	Program = { NULL, NULL,  VERSUPV, VERSUPR, VERSUPT };
//const	char	PROGNAME[] = VERSUPT;
// mgutils expect PROGNAME and VERSION strings to exist as global labels

typedef struct  {
	int	dbug;	// print level
	int	star;	// show star detail
	int	dvfd;	// show file data
// add features here
//	int	wvfd;	// Write compiled data to a file
	}  opts;


/*  These defines are from skymap.c as needed.  */


//#define  HeaderForm   "%d %10.7f %10.6f %8.3f %5d\n"
#define  HeaderFormI  "%d  %lf    %lf    %lf   %d"
#define  CL_header  32
// HeaderForm contains cat, alpha, delta, frad, stars
//#define  CacheForm  "%10lu %10.7f %10.6f %5.2f %.8s\n"
#define  CacheFormI "%lu   %lf    %lf    %lf   %s\n"
#define  CL_star  44
// The header and star lengths are just estimates now
// CacheForm contains id, alpha, delta, mag, spc
#define  CacheBUFL 127


/*  ===  Utility Programs  ===  */


	/* - - - - - - - - - - - - - - *\
	|				|
	|	Test Subroutines	|
	|				|
	\* - - - - - - - - - - - - - - */


/*  ===  test subroutine here  ===  */


/*  ==  Actual application subroutines  ==  */


	/* - - - - - - - - - - - - - - *\
	|				|
	|    Reporting  Subroutines	|
	|				|
	\* - - - - - - - - - - - - - - */


static	void	multline (int* a, int* b, char* m1, char* m2, int w)
/*  Multi line support program -- could be generalized.
  Write either "%d: m1" or "%d - %d: m2", depending on *a, *b, w
  and update *a and *b.  w is 1 to write, 0 to accumulate.
  For the w == 0 case:
    If b < 0, set a = b, return.
    If a < 0, set a = b, return.
    Otherwise, return.
  For the w == 1 case:
    If b < 0, set a = b, return.
    If a == b, write m1 and set b=a=-1, return
    If a < b, write m2, set b=a=-1, return
    Otherwise write m1 (b), set b=a=-1, return  
  */
/*  Usage:
  Start with b = -1 and a = -1.
  When adding a line to be written, set b = x and call with w = 0.
  Before writing a different line, call with w = 1.
  At end, call with w = 1.
 */
{
    if (*b < 0) goto out2;
    if (w == 0) {	// Save, but don't write yet.
	if (*a < 0) goto out2;
	return;
    } else {
	if (*a < *b && *a >= 0) {
	    printf ("%2d - %d: %s\n", *a, *b, m2);
	} else {
	    printf ("%2d: %s\n", *b, m1);
	}
    }
// out1:
    *b = -1;
out2:
    *a = *b;
}



char*   cat_name (int cat_id)
/*  Return the catalog name coresponding to the cat_id integer.
  The symbolic names are defined in catalog.h  */
{
static  char    *c;
    switch (cat_id) {
	case CAT_SAO:           c = "SAO";              break;
	case CAT_PPM:           c = "PPM";              break;
	case CAT_GSC:           c = "GSC";              break;
	case CAT_GSCP:          c = "GSCP";             break;
	case CAT_USNO_SA:       c = "USNO_SA";          break;
	case CAT_USNO_AO:       c = "USNO_AO";          break;
	default:                c = "Unknown";          break;
    }
    return  c;
}



	/* - - - - - - - - - - - - - - *\
	|				|
	|      Process Subroutines	|
	|				|
	\* - - - - - - - - - - - - - - */

/*  ===  Main Process is put here  ===  */



int	processor (char* filename, opts opt)
/* Process the input file named.  Return error count. */

/* Write a report on the input unless the -v or -z flags are
  in use; do compare to other file unless -c is in use.  */
{
int	r = 0;		// return status
int	k = 0;		// error count
int	ic = 0;		// input count (debug at least)
int	j = 0;
int	n = 0;
FILE	*f = NULL;
char	line[CacheBUFL+1];
char	*spec = NULL;
char	fnam[CacheBUFL+1];
char	filex[CacheBUFL+1];
char	afld[32];
char	dfld[32];
int	cat;
double	alpha, delta;
double	frad;
double	mag;
int	stars = 0;
unsigned long	id;
char	sp[10];



/* Obtain the actual file name to use */
    if (filename == NULL) {
//	printf ("  No file name error.\n");
//	k++;

char	*dptr;
char	*directory;
char	*dxf;
int	i;
int	a, b;
int	t=0;
char	m1[] = "exists.";
char	m2[] = "exist.";
char	x1[] = "has no entry.";
char	x2[] = "no entry.";
/* If no file name, try to find the directory (error if can't), and
  then special report if it is /dev/null, and then report on the
  cache files found in the directory.  Then go to errorexit.  */
	dptr = getenv ("CATCACHE");
	directory = strdup (dptr ? dptr : "./");
	if (opt.dbug > 2) printf (" *= Cache directory \"%s\".\n", directory);
	if (access (directory, R_OK) ) {  // problem if not readable
	    printf (" ** Directory \"%s\" not readable.\n", directory);
	    k++;
	} else {	// directory itself is OK
/* Seems that /dev/null checks out OK, so we need to test for it directly. */
	    if (!strcmp (directory, "/dev/null")) {
		printf (" *= Cache directory \"%s\" inhibits caching.\n",
			directory);
		k++;
	    } else {  // Good directory, check the files
		printf (" *= Using Cache directory \"%s\".\n", directory);
// If not detail, just report existance in a loop...
	      if (opt.dvfd == 0) {	// No details on cache files
		for (i=0,a=b=-1; i<16; i++) {  // could extend the loop...
		    sprintf (fnam, "OCIW_cat%c.dat", exchar(i));
		    dxf = fncat (directory, fnam);  // dynamic
		    if (access (dxf, F_OK) == 0) {	// exists
//			printf (" %s exists.\n", fnam);
			b = i;
			t++;
			multline (&a, &b, m1, m2, 0);
		    } else {				// does not exist
			multline (&a, &b, m1, m2, 1);
		    }
		    free (dxf);
		}
		multline (&a, &b, m1, m2, 1);
// else if detail, run a loop to report all existing files and multiline non-entries
// as is done in skymap
	      } else {  // cache file detail -v option
		for (i=0,a=b=-1; i<16; i++) {  // could extend the loop...
		    sprintf (fnam, "OCIW_cat%c.dat", exchar(i));
		    dxf = fncat (directory, fnam);  // dynamic
		    if (access (dxf, F_OK) == 0) {	// exists
			multline (&a, &b, x1, x2, 1);
// Read the details from the file and report
			f = xopen (dxf, "r");	 // opens file
			fgets (line, CacheBUFL, f);  // reads header
			j = sscanf (line, HeaderFormI, &cat,
				&alpha, &delta, &frad, &n );
// Not checking for j < 4 errors here unless needed
/* Note that some data might not have star count.  */
			stars = (j >= 5) ? n : 0;
			printf ("%2d: %s  Cat: %s\n", i, dxf, cat_name(cat) );
			sexigw (afld, alpha,  3, 3);
			sexigw (dfld, delta, -3, 2);
			printf ("  : R.A. %s  Dec. %s  Rad. %7.2f, %6d stars.\n",
				afld, dfld, frad, stars);
			iclose (f);    f = NULL;

// Report the index: filename cat: xxx /n ra xx:xx:xx  dec xx:xx:xx, rad xx.x, nn stars
//			b = i;
			t++;
//			multline (&a, &b, m1, m2, 0);
		    } else {				// does not exist
			b = i;
			multline (&a, &b, x1, x2, 0);
		    }
		    free (dxf);
		}  // end of if on -v option
		multline (&a, &b, x1, x2, 1);

	      }
		if (t > 0) printf (" Total of %d cache files exist.\n", t);
		else printf (" No cache files found.\n");
	    }
	}
	if (access (directory, R_OK + W_OK) ) {  // warn if not read/write
	    printf (" ** Warning: Directory \"%s\" NOT writable.\n", directory);
	}
	free (directory);

/* =====  */

/* Get the directory, verify that it is proper.  If not, set skip status
  and return without loading index.  Initialize directory name.      */
/* Check the directory.  If read/write it should be good.  Otherwise, it
  is bad and we skip caching.  The "/dev/null" is special and indicates
  no caching is to be attempted.  Even if it is attempted, it would not
  work, but the index would get inproperly filled with data which was
  not written, wasting effort.    */
/* things borrowed from skymap.c:830  */


/* =====  */


// End of null filename case here...
    }
    if (k > 0 || filename == NULL) goto errorexit;

//    if (filename == NULL) source = VERSFIL;
//    else		  source = filename;
    if (opt.dbug > 0 && k == 0) printf (" Input File name: %s\n", filename);

/* Write compiled data to file request */
//    if (opt.wvfd) {
//	printf (" Output to file -v requested.\n");
//	return  k;
//    }

/*  See if file exists.  Try as a name, then try as partial name
  adding .dat and prepending OCIW_cat to find file.  Give message
  if file not found.  Otherwise, report on the file.  Use findfile.
  Recognize CATCACHE env.vbl.  */

#ifdef  LOCATE
// First try uses name and name with OCIW_cat prepended, using a
// suffix list with null, .dat;
// Second try uses uppercase name and if possible numeric name
// converted to a character with suffix list of .dat only.
// Directory list is null and CATCACHE both times.
    sprintf (fnam, "%s,OCIW_cat%s", filename, filename);
    spec = locfile ("r", fnam, ",.dat", ",CATCACHE");
    if (spec == NULL) {
	strncpy (filex, filename, CacheBUFL);
	upcase (filex);		// try upper case
	j = sscanf (filex, "%d", &n);
	if (j > 0 && n < 63) {  // check for numeric argument
	    sprintf (fnam, "OCIW_cat%s,OCIW_cat%c", filex, exchar(n));
	} else {
	    sprintf (fnam, "OCIW_cat%s", filex);
	}
/* An alternative to the above is to use "OCIW_cat" as the name, and
  for sufx use {filex}.dat and append ,{exchar(n)}.dat if j looks good.
	sx = strdup (filex);
	sx = xcat (&sx, ".dat");
	if (j ... ) {
	  sx = xcat (&sx, ",");
	  sx = xcat (&sx, exchar(n))
	  sx = xcat (&sx, ".dat");
	}
...	locfile ("r", "OCIW_cat", sx, ",CATCACHE")
	(that is 2 lines shorter)      */
// exchar is in kdcutil
	spec = locfile ("r", fnam, ".dat", ",CATCACHE");
    }
#else
// First, try the name itself, and with .dat added
    findfile (&spec, "r", filename, NULL);
    findfile (&spec, "r", filename, "CATCACHE");
    sprintf (fnam, "%s.dat", filename);
    findfile (&spec, "r", fnam, NULL);
    findfile (&spec, "r", fnam, "CATCACHE");

// See if substituting as a number will work...
// (The first part here will probably not be useful.
    sprintf (fnam, "OCIW_cat%s", filename);
    findfile (&spec, "r", fnam, NULL);
    findfile (&spec, "r", fnam, "CATCACHE");
    sprintf (fnam, "OCIW_cat%s.dat", filename);
    findfile (&spec, "r", fnam, NULL);
    findfile (&spec, "r", fnam, "CATCACHE");

// See if substituting as upper case is what is needed
// NOTE -- Do not do this if extended (over 16) indexing is in use!!
    strncpy (filex, filename, CacheBUFL);
    upcase (filex);		// try upper case
    sprintf (fnam, "OCIW_cat%s.dat", filex);
    findfile (&spec, "r", fnam, NULL);
    findfile (&spec, "r", fnam, "CATCACHE");

// See if the argument is a number which needs to be converted to character
    j = sscanf (filex, "%d", &n);
    if (j > 0 && n < 63) {	// try numeric argument
	sprintf (fnam, "OCIW_cat%c.dat", exchar(n) );  // (in kdcutil)
	findfile (&spec, "r", fnam, NULL);
	findfile (&spec, "r", fnam, "CATCACHE");
    }
#endif

// Out of ideas, if not found yet an error message is needed.

    if (spec == NULL) {   // Could not find it
	printf (" ** Could not find cache file \"%s\".\n", filename);
	k++;
	goto errorexit;
//    } else if (opt.dbug > 0 && k == 0) {
//	printf (" File used is:  %s\n", spec);
    }

/*  Open the file and read the first record.  Try to decode it.  Give
  error message if bad format.  If good, report catalog name, the
  center and radius and star count.  */

    f = xopen (spec, "r");	 // opens file
    fgets (line, CacheBUFL, f);  // reads header
    j = sscanf (line, HeaderFormI, &cat,
	&alpha, &delta, &frad, &n );
    if (j < 4) {
	printf (" ** Header of %s has only %d values.\n", spec, j);
	k++;
	goto errorexit;
    }
    if (j >= 5) stars = n;  // some data may not have star count
    printf (" Cache file:   %s \n", spec);
//  printf ("    Catalog:   %d\n", cat);  // replace by name!
    printf ("    Catalog:   %s\n", cat_name(cat) );
    sexigw (afld, alpha,  3, 3);
    sexigw (dfld, delta, -3, 2);
    printf ("   Centered:   %s  %s\n", afld, dfld);
    printf ("     Radius:   %7.2f\n", frad);
    if (stars > 0) printf (" Star count:   %5d\n", stars);
    if (opt.star)
      printf ("-- I.D. --      R.A.           Dec.      Mag. Spect.\n");

/*  Loop to report the file content.  Will have already returned an error
  status if previously detected.  Report all stars to stdout.  */

    while (fgets (line, CacheBUFL, f) != NULL) {
	*sp = '\0';
	j = sscanf (line, CacheFormI, &id, &alpha, &delta, &mag, sp);
	trimend (line, NEWLINES);
	if (j < 4) continue;
	sexigw (afld, alpha,  3, 3);
	sexigw (dfld, delta, -3, 2);
	if (opt.star) {
// An alternate to not printing is to set *sp to '\0' as in null, above.
//	if (j < 5)
//	    printf ("%10lu  %s  %s  %5.2f\n", id, afld, dfld, mag);
//	else
	    printf ("%10lu  %s  %s  %5.2f  %s\n", id, afld, dfld, mag, sp);
	}  // option on displaying stars
	ic++;
    }
    printf (" Stars found:  %5d\n", ic);

    iclose (f);    f = NULL;

#ifdef  NOTDEFINEDATALL
    if (filename) f = xopen (filename, "r");
    if (filename && !f) {
	sprintf (line, "Opening \"%s\"", filename);
	perror (line);
	k++;
    }
    while (f && fgets (line, 120, f) != NULL) {
	ic++;
// clean end and beginning of the line
// for begin, use strspn (buf, whitespace) to find any initial blanks.
// Drop lines starting with ! or #, or that are null
// data in line, decode obs/smf/output items (use parse)
// for .obs read the file to Obs with read_obsfile (clear first)
// For smf, read with read_smdf (clear first)
// for output, open the file and save it OK
// if output was previously open, delete it (unlink)
// if line is not decoded, push the data to namlist queue
    }	// end of input reading loop
    if (f) iclose (f);
    f = NULL;
    if (opt.dbug > 1) printf (" ..%d lines read from input.\n", ic);

/* Here's what we do:
  -- Try to open the file named; if fail that's an error
  and we give an error message.  Otherwise, read the file.
  -- In a loop, read all the lines in the file.  Test for
  the output file spec, and the .SMF or .obs file spec.
  If found, read that file to the Obs structure; but
  kill any previous Obs first.  In case of output file,
  just save the file spec.  If a comment, ignore.
  For all other lines, queue it in a namlist queue.
  -- At end of loop, close the input file.
  -- Check if we got a Obs, if not that's an error
  and we count it and report.
  -- Open output file.  If none, or error, count the
  error, and report.
  -- If we have an error count, return.
  -- If no errors, pass the Obs and all other lines
  read on to the output processor.
  -- Close the output file upon return.
  -- Count all the errors found.  If any, return 1,
  and if none, return 0.  If errors, give a message
  referencing the input file name.
  -- Clear storage at obs and head.
  -- That should be it.  Done.		*/

/* Parsing routines:
In string.h  (POSIX, ISO9899)
    char* strchr (char *s, int c); strrchr (same)  find character left/right
    int strspn (char *s, char *a); strcspn (same)  Num char in a, not in a
    char* strstr (char *s, char *a); find string a in string s
    char* strpbrk (char *s, char* a);  Find first loc in s of char in set a
    char* strsep(char **s, char* d); return token delim by d in s, modify s
    char* strtok (char *s, char* d); like strstp, modifies s.  DO NOT USE
    int strcmp (char *s1, char *s2); compare strings
    char* strdup (char *s); return dynamic string copy of s (as dynstr)

In strings.h  (4.3BSD)
    char* index (char *s, int c); rindex (same)  find char left/right (strchr)
    int strcasecmp (char *s1, char *s2); compare strings case insensitive

GNU extensions
    void *memfrob (void *s, size_t n); x-or s with 42 in each byte.
    int strverscmp (char *s1, char *s2); compare strings as version numbers

My parsing
    char* parse (char* line, char* tok, char* delim) copy first token found
      to tok, return pointer to remainder of line.
    char* getok (char** line, char* delim) return dynamic string containing
      delimited token from line, advance line pointer.  Line unmodified.

Consider #define _GNU_SOURCE before include of string.h for several
extensions such as strcasestr and strverscmp.  But that's non-portable,
and will not work with Mac-OS unix for example.
See "man string" for a list of most of these.		*/


#endif  // end of dropped section

errorexit:
/* Return any status derived */
    r = k ? 1 : 0;
    if (k) {
	printf (" - Found %d errors processing %s\n", k,
		filename ? filename : "Null" );
    }
    return  r;
}


	/* - - - - - - - - *\
	|		    |
	|    Main Program   |
	|		    |
	\* - - - - - - - - */

/*  ====  MAIN PROGRAM  ====  */

#define  PROMPTLOOP
//#undef  PROMPTLOOP

#define  LOOPMETHODUSEDASOLDWAY
//#undef  LOOPMETHODUSEDASOLDWAY

int  main (int argc, char** argv) {
extern char *optarg;
extern int optind,opterr,optopt;
int c;
int     dohelp = 0;
// The following symbol is nominally undefined
#ifdef  LOOPMETHODUSEDASOLDWAY
char	infile [120];	/* For prompting */
#endif
opts	opt;
namlist	*head = NULL;
char	*filename = NULL;
#ifdef  LOOPMETHODUSEDASOLDWAY
int	k, m, n;	/* Process counts */
#else
int	k = 0;
#endif
int	xx = 0;		/* Exit flag */

/* Usual flags, debug and help */

/* Standard input logic:  If argument given, take that argument as the
  sole specification to process.  If no argument is given, prompt for
  an argument, and repeat the prompting until a null reply.  */

    opt.dbug = 0;	/* Or - put data in define above */
    opt.star = 0;	/* Omit star detail */
    opt.dvfd = 0;	/* No detail file data */
//    opt.wvfd = 0;	/* Write version file data */

/* Decode all the options of the command line */
/* :: follows letter with optional argument, : follows one with required
  argument, otherwise there are no arguments to the options.  */
    while((c = getopt(argc, argv, "d::qsvh")) != -1) switch(c) {
	case 'd':				// debug level
	    opt.dbug = dodebug = optarg == NULL ? 1 : atoi (optarg);	break;
	case 'q':	xx = 1;		break;	// display version and exit
	case 's':	opt.star = 1;	break;	// add star display
//	case 'v':	opt.wvfd = 1;	break;	// write file
	case 'v':	opt.dvfd = 1;	break;	// show file data
	case 'h':	dohelp = 1;	break;	// parameter help
	default:
	    printf (" unused %c option\n", c);	break;
    }  /* end of switch, only statement in the getopt while loop */

//    dodebug = opt.dbug;

    for (c = optind; c < argc; c++) {
	if (filename == NULL) filename = dynamstr (argv[c]);
	head = pushtxt (argv[c], head);  /* Collect arguments */
//	if (opt.dbug > 0) fprintf (stderr, "-Argument %s.\n", filename);
	if (opt.dbug > 0) fprintf (stderr, "-Argument %s.  ", argv[c]);
/* NOTE that dynamstr must be used to put any static string into a namlist,
  usually in  get_nam, and the like.  It will cause a crash at the
  end when the name queue is dropped, if this is not done correctly.
  That is why pushtxt is used above.  */
    }


    if (dohelp) {
	printf ("\n %s -- Version %s", PROGNAME, VERSION);
	printf ("\n  Parameters are: ");
	printf ("\n     Input File name or number(s) ");
	printf ("\n   (Null file to report which files exist)");
//	printf ("\n     (Default environment name is CATCACHE)");
	printf ("\n   (Cache directory is from CATCACHE or ./");
	printf ("\n  Switches");
	printf ("\n    -d* = Set debug level to (*) [1]");
	printf ("\n     -h = Display this help");
	printf ("\n     -q = Display version and exit");
	printf ("\n     -s = Show star detail");
//	printf ("\n     -v = Write current data to file");
	printf ("\n     -v = Show content of existing files");
	printf ("\n   Reports cache file on stdout.");
/*	printf ("\n   Output file name is prompted for."); */
	printf ("\n \n");
	return (0);
    }

    if (opt.dbug)
	printf ("\n %s -- Version %s\n", PROGNAME, VERSION);
//    if (opt.ncmp) opt.tfil = 0;  /* Holdover from old meaning of tfil */
    if (xx) return  0;	// Implement -q

/*  -- Loop to process arguments, or prompt if no arguments -- */
#ifdef  LOOPMETHODUSEDASOLDWAY
    for (k=n=m=0; k >= 0; ) {	// Forever loop
	if (head == NULL) {	// No arguments, or looped out
#ifdef  PROMPTLOOP
	    if (m > 0) break;	// Had real arguments, looped out
#else
	    if (n > 0) break;	// No loop after any counted argument
#endif
//-	  if (n == 0)
	    askmestr (infile, 110, 
		n==0 ? "Input file: [NULL for all]" : "Input file: ");
//-	  else
//-	    askmestr (infile, 110, "Input file: ");
//	    if (*infile == '\0') break;	// Exit on any null reply
	  if (*infile != '\0')
	    filename = dynamstr (infile);
	  else if (n > 0) break;  // break if null reply after counted arg.
	  else m++;  // Count a null reply as a given argument
	} else {		// We have a command line argument given
	    filename = pop_name (&head);
	    if ( filename == NULL) continue;
	    if (*filename == '\0') continue;
	    m++;	// Count number of command line arguments given
	}
	n++;		// Count number of arguments requested

	k += processor (filename, opt);  // was  k++;
	free (filename);	// MUST be dynamic
	filename=NULL;
    }
#else
    k = processor (filename, opt);
#endif
    if (k > 0) printf (" ** Total of %d errors detected.\n", k);

/* Following dummy statement to avoid warning messages...  */
    if (cat_equ[0] == cat_mag[0]) ;

/* Free dynamic storage used for version field and filename. */
    free (filename);
    return  k;
}

/*  Utility program template.
  Needs a file name, and will prompt for it if not present.
  May process several file names given as inputs.
  Shows how to write a processing subroutine.
  */

/* Some change notes

    10/10-05 - Version 0.29
	Add environment name to help display
	Add -v option to basic default cache display
	11/1-6 add "NULL for all" to file prompt

    10/08-20 - Version 0.28
	Add default report on which cache files exist,
	and change -s to report star listing.

    10/08-16 - Version 0.25
	Add -s flag to omit detail star listing

    10/01-12 - Version 0.20
	Add numeric decode features
	Some improvement to output format
	
    10/01-07 - Version 0.10
	Initial version.

.. */

