/* ----------------------------------------------------------------------
 *
 * catalog.c
 *
 * star-catalog functions
 *
 * Project:  GMAP
 *
 * Christoph C. Birk (birk@ociw.edu) , OCIW, Pasadena, CA
 *
 *  Added some GSC-plus stuff  KDC
 *  Changed catalog alpha,delta from float to double
 *  Added vector value to catalog lookup of alpha, delta
 *  2005/12-7 -- added 	/usr/local/etc/gcats  catalog location
 *  2011/4-7 -- Allow CATPATH to be path formatted list
 *
 * ---------------------------------------------------------------------- */

#define IS_CATALOG_C

/* --------DEFINEs ------------------------------------------------------ */

/* #define DEBUG */

#define SAO_RECORD_SIZE   15
#define GSC_RECORD_SIZE   8
#define PPM_RECORD_SIZE   12
#define USNO_RECORD_SIZE  12
#define OCIW_RECORD_SIZE  8

// #define RADS              57.29577951         /* 180/pi */

/* TYPEDEFs ------------------------------------------------------------- */


/* INCLUDEs ------------------------------------------------------------- */

#include <stdlib.h>                           /* abs() */
#include <stdio.h>
#include <string.h>                           /* strcpy() */
#include <math.h>                             /* cos(),floor()*/
#include <unistd.h>		/* Bits in access, e.g. R_OK */

#include "catalog.h"	// includes kdcutil.h
#include "utils.h"
#include  IOUTILH	// needed for parse
#include  FEATSH

/* Globals... */
extern	int	TESTMODE;

/* STATICs -------------------------------------------------------------- */

static char* cat_cat[]={"sao.cat","ppm.cat","gsc.cat","gscp.cat",
                        "usno-s.cat","usno-a.cat",""};
static char* cat_idx[]={"sao.idx","ppm.idx","gsc.idx","gscp.idx",
                        "usno-s.idx","usno-a.idx",""};
static char  sp0[]=" OBAFGKM";                /* spectral types */
static FILE  *cp=NULL;
static int   a1[480][360],a2[480][360]; /* todo dynamical allocation */

/* ---------------------------------------------------------------------- */

char*	catpath (void)
// Utility to find and return directory path to star catalogs
{
#ifdef  NOCATS
char	*rv;
#else  // i.e. #ifndef  NOCATS
//static	char	de[2][12] = { "GMAPCATS", "CATPATH" };
static	char	de[2][12] = { "CATPATH", "GMAPCATS" };
static	char	dp[2][12] = { "./CATS", "./gcats" };
static	char	ds[32] = { "/usr/local/etc/gcats" };
/* Here we have a debug feature to simulate no catalog. */
//static	char	ds[32] = { "/dev/null" };
static	char	buf[256];
static	char	par[256];
int	i;
#endif  // on NOCATS to drop warning messages

/* Replaces the former call which was:
	genv3("GMAPCATS","CATPATH","../CATS")
  with a unified search method.  */
/* This searches for the catalog files in the same way as the previous
  version, with the addition of looking in local directory ./gcats,
  and trying GMAPCATS and CATPATH as local directories, too.  Other
  possible directory pointers may be added easily.  */

/* The preferred method for defining catalog files is to use the
  environment variable GMAPCATS to define the catalog directory.
  Otherwise, establish a symbolic link at ../CATS or locally in
  ./gcats (or possibly ../gcats).  */

/* One exception for better use at OCIW; temporarily we look for
  the usno catalog in the env.vbl. returned for GMAPCATS, and don't
  use it if this catalog is not present. */

/* When debugging, use only the CATPATH value. */
#ifdef  NOCATS
    rv = getenv ("CATPATH");
    if (rv == NULL) return "/dev/null";
    else	return  rv;
#else  // on NOCATS
/* First, try the initial two getenv values as before. */
    for (i=0; i<2; i++) {
char	*c;	// scratch, local
//	rv = getenv (de[i]);
//	if (rv != NULL) {	// Temporary bypass for OCIW internals...
//	    sprintf (buf, "%s/usno-o", rv);
//	    if (!access(buf, R_OK)) return rv;
/* New Feature!  Having rv, in a loop we parse it into tokens with delimiters
  of ":" or " " and try each one as a possible directory, returning the first
  one which works.  This allows CATPATH for example to be a path formatted
  xx:xx list of directories, and a simple directory value is a valid 
  subset of this form.  Use "parse" from ioutils to parse it.    */
	for (c = getenv(de[i]); c != NULL; ) {
	    c = parse (c, par, ": ");
// par must be large enough to hold any token CATPATH can deliver
	    if (strlen(par) < 1) break;
	    sprintf (buf, "%s/usno-o", par);
	    if (!access(buf, R_OK)) return  par;
	}
/* The above section is an exception for OCIW.  The GMAPCATS environment
  is often defined for local users to point to a directory which has been
  deficient in the usno catalog.  So, if we don't find it, we will not
  use that directory at all, and try the others.  */
    }

/* Try a couple of default pointers to see if they work */
    for (i=0; i<2; i++) {		// Try first in parent directory
	sprintf (par, ".%s", dp[i]);
	sprintf (buf, "%s/usno-o", par);
	if (!access(buf,R_OK)) return par;
    }

    for (i=0; i<2; i++) {		// Try them in local directory
	sprintf (buf, "%s/usno-o", dp[i]);
	if (!access(buf,R_OK)) return dp[i];
    }

/* Add standard location of /usr/local/etc/gcats */
    sprintf (buf, "%s/usno-o", ds);
    if (!access(buf,R_OK)) return ds;

/* All out of ideas, return our usual default */
//    sprintf (par, ".%s", dp[0]);
    sprintf (par, "%s", ds);
    return  par;
#endif  // on NOCATS
}

int cat_open(int c)
{
  int  i,j;
  char path[256],catdir[256];
  FILE *fp;

// printf (" >> cat_open called for %d from %d.\n", c, catalog);

  strcpy(catdir,catpath( ));
  sprintf(path,"%s/%s",catdir,cat_idx[c]);
#ifdef DEBUG
  fprintf(stderr,"path=%s\n",path);
#endif
  if (TESTMODE) fprintf (stderr, " Catalog path = \"%s\"\n", catdir);

  switch (c) {                       /* load index to catalog */
  case CAT_SAO:                      /* SAO  */
    if ((fp=fopen(path,"r")) == NULL) return 0;
    for(j=0;j<18;j++){               /* delta-bandwidth = 10 deg */
      for(i=0;i<24;i++){             /* hour-bandwidth = 1 h */
        if (fscanf(fp,"%*d %*d %d %d",&a1[i][j],&a2[i][j]) < 2) { 
          fclose(fp); return 0; 
        }
      }
    }
    fclose(fp);
    sprintf(path,"%s/%s",catdir,cat_cat[c]);
    if ((cp=fopen(path,"rb")) == NULL) return 0;  /* failed */
    break;
  case CAT_USNO_SA:                  /* USNO-SA2.0 */
    for (j=0; j<24; j++) {           /* delta bandwidth = 7.5 deg */
      sprintf(path, "%s/usno-sa/zone%04d.acc", catpath( ),j*75);
      if ((fp=fopen(path,"r")) == NULL) return 0;
      for (i=0; i<96; i++) {         /* hour bandwidth = 0.25 h */
        if (fscanf(fp,"%*f %d %d",&a1[i][j],&a2[i][j]) < 2) {
          fclose(fp); return(0);
        }
        a2[i][j] += a1[i][j]-1;
      }
      fclose(fp);
    }
    cp = NULL;
    break;
  case CAT_USNO_AO:                  /* USNO-A2.0 (OCIW) */
    {
      int total=0;
      for (j=0; j<360; j++) {          /* delta bandwidth = 0.5 deg */
        sprintf(path, "%s/usno-o/zone%04d.idx", catpath( ), j*5);
        if ((fp=fopen(path,"r")) == NULL) return 0;
        for (i=0; i<480; i++) {        /* hour bandwidth = 0.05 h */
          if (fscanf(fp,"%d %d",&a1[i][j],&a2[i][j]) < 2) {
            fclose(fp); return(0);
          }
          total += a2[i][j];
          a2[i][j] += a1[i][j]-1;
      }
        fclose(fp);
      }
#ifdef DEBUG
      fprintf(stderr,"total=%d\n",total);
#endif
    }
    cp = NULL;
    break;
  case CAT_GSCP:                     /* GSCplus */
    {
      int total=0;
      for (j=0; j<72 ; j++) {        /* delta bandwidth = 2.5 deg */
        sprintf(path, "%s/GSCplus/gscp%04d.idx", catpath( ), j*25);
        if ((fp=fopen(path,"r")) == NULL) return 0;
        for (i=0; i<96; i++) {        /* hour bandwidth = 0.25 h */
          if (fscanf(fp,"%d %d",&a1[i][j],&a2[i][j]) < 2) {
            fclose(fp); return(0);
          }
          total += a2[i][j];
          a2[i][j] += a1[i][j]-1;
        }
        fclose(fp);
      }
#ifdef DEBUG
      fprintf(stderr,"total=%d\n",total);
#endif
    }
    cp = NULL;
// dummy statement to avoid warnings
if (cat_equ[c] == cat_mag[c]) ;
    break;
  case CAT_GSC:                      /* GSC */
  case CAT_PPM:                      /* PPM */
    if ((fp=fopen(path,"r")) == NULL) return 0;
    for (j=0; j<24; j++) {           /* delta-bandwidth = 7.5 deg */
      for (i=0; i<24; i++) {         /* hour-bandwidth = 1 h */
        if (fscanf(fp,"%d %d",&a1[i][j],&a2[i][j]) < 2) {
          fclose(fp); return 0;
        }
      }
    }
    fclose(fp);
    sprintf(path,"%s/%s",catdir,cat_cat[c]);
    if ((cp=fopen(path,"rb")) == NULL) return 0;  /* failed */
    break;
  default:
    return 0;
  }

  catalog = c;                       /* set 'catalog' */

  return(1);
}

/* ---------------------------------------------------------------------- */

void cat_close(void)
{
  if (cp) { fclose(cp); cp=NULL; }
}

/* ---------------------------------------------------------------------- */

void cat_ibound(double a0,double d0,double rx,double ry,int* id1,int* id2,int* ia1,int* ia2)
{
  double cosd = 15.*3600.*cos(d0*Degree);       /* local 'cos(delta)' */

  switch (catalog) {                          /* delta indices */
  case CAT_SAO:                               /* index-boundaries for SAO */
    *id1 = (int)floor((90.-d0-ry/3600.)/10.); /* delta b-width = 10 */
    if (*id1 > 17) *id1=17;
    *id2 = (int)floor((90.-d0+ry/3600.)/10.);
    if (*id2 > 17) *id2=17;
    break;
  case CAT_USNO_SA:                           /* index boundaries USNO-SA */
    *id1 = (int)floor((d0-ry/3600.+90.)/7.5); /* delta b-width = 7.5 */
    if (*id1 > 23) *id1=23;
    *id2 = (int)floor((d0+ry/3600.+90.)/7.5);
    if (*id2 > 23) *id2=23;
    break;
  case CAT_USNO_AO:                           /* index bound. USNO-A-OCIW */
    *id1 = (int)floor((d0-ry/3600.+90.)/0.5); /* delta b-width = 0.5 */
    if (*id1 > 359) *id1=359;
    *id2 = (int)floor((d0+ry/3600.+90.)/0.5);
    if (*id2 > 359) *id2=359;
    break;
  case CAT_GSCP:                              /* index boundaries for GSCplus */
    *id1 = (int)floor((d0-ry/3600.+90.)/2.5); /* delta b-width = 2.5 */
    if (*id1 > 71) *id1=71;
    *id2 = (int)floor((d0+ry/3600.+90.)/2.5);
    if (*id2 > 71) *id2=71;
    break;
  case CAT_GSC:                               /* index-boundaries for GSC */
  case CAT_PPM:
    *id1 = (int)floor((90.-d0-ry/3600.)/7.5); /* delta b-width = 7.5 */
    if (*id1 > 23) *id1=23;
    *id2 = (int)floor((90.-d0+ry/3600.)/7.5);
    if (*id2 > 23) *id2=23;
    break;
  default:
    *id1 = *id2 = 0;
    break;
  }
  if (*id1 <  0) *id1=0;
  if (*id2 <  0) *id2=0;
#ifdef DEBUG
  fprintf(stderr,"id1=%d, id2=%d\n",*id1,*id2);
#endif


  switch(catalog) {                           /* alpha indices */
  case CAT_USNO_SA:                           /* USNO-SA2.0 */
  case CAT_GSCP:                              /* GSCplus */
    *ia1 = (int)floor(4.*(a0-rx/cosd));       /* 0.25 h wide */
    *ia2 = (int)floor(4.*(a0+rx/cosd));
    if (*ia2 - *ia1 > 95)  { *ia1 = 0; *ia2 = 95; }
    break;
  case CAT_USNO_AO:                           /* USNO-A2.0 (OCIW) */
    *ia1 = (int)floor(20.*(a0-rx/cosd));      /* 0.05 h wide */
    *ia2 = (int)floor(20.*(a0+rx/cosd));
    if (*ia2 - *ia1 > 479) { *ia1 = 0; *ia2 = 479; }
    break;
  case CAT_SAO:                               /* SAO catalog */
  case CAT_GSC:                               /* GSC */
  case CAT_PPM:
    *ia1 = (int)floor(a0-rx/cosd);
    *ia2 = (int)floor(a0+rx/cosd);
    if (*ia2 - *ia1 > 23)  { *ia1 = 0; *ia2 = 23; }
    break;
  default:
    *ia1 = *ia2 = 0;
    break;
  }
#ifdef DEBUG
  fprintf(stderr,"ia1=%d, ia2=%d\n",*ia1,*ia2);
#endif
}

/* ---------------------------------------------------------------------- */

void cat_indices(int i,int j,int* n1,int* n2)
{
  switch (catalog) {
  case CAT_SAO:
  case CAT_GSC:
  case CAT_PPM:
    while (i < 0)   i += 24;
    while (i > 23)  i -= 24;
    break;
  case CAT_USNO_SA:
  case CAT_GSCP:                   /* GSCplus */
    while (i < 0)   i += 96;
    while (i > 95)  i -= 96;
    break;
  case CAT_USNO_AO:
    while (i < 0)   i += 480;
    while (i > 479) i -= 480;
    break;
  }
  *n1 = a1[i][j];
  *n2 = a2[i][j];
}

/* ---------------------------------------------------------------------- */

int GetSAO(long id,CATstar *star,double epoch)
{
  int        nseek;
  double     ua,ud;
  u_char     buf[16];
  static int oseek=-1;                           /* old seek position */
   
  nseek = (id-1)*SAO_RECORD_SIZE;                /* new seek position */
  if (nseek-oseek != SAO_RECORD_SIZE) fseek(cp,nseek,0); /* seek */
  oseek=nseek;                                   /* old seek */
  (void)fread((void*)buf,SAO_RECORD_SIZE,1,cp);  /* read record */
 
  star->id    = id;
  star->alpha =  (((double)buf[3]*65536.+(double)buf[4]*256.+(double)buf[5])/360000.);
  star->delta = (((double)buf[6]*65536.+(double)buf[7]*256.+(double)buf[8]-3240000.)/36000.);

  if (buf[9]==255) star->mag = (float)99.9;
  else             star->mag = (float)(buf[9]/10.);
 
  star->spc[0] = sp0[buf[10]/16];                /* higher 4 bit */
  star->spc[1] = '0' + (buf[10] & (u_char)0x0f); /* lower 4 bit  */
  star->spc[2] = '\0';
 
  ua = ((double)buf[11]*256. + (double)buf[12] - 32768.)/10000.;
  ud = ((double)buf[13]*256. + (double)buf[14] - 32768.)/1000.;
 
  star->alpha += (double)(ua/3600. * (epoch-1950.));
  star->delta += (double)(ud/3600. * (epoch-1950.));
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);
 
  return 1;
}
 
/* --------------------------------------------------------------------- */

int GetGSC(long id,CATstar* star)
{
  int        nseek;
  u_char     buf[16];
  static int oseek=-1;                          /* old seek position */
 
  nseek = (id-1)*GSC_RECORD_SIZE;               /* new seek position */
  if (nseek-oseek != GSC_RECORD_SIZE) fseek(cp,nseek,0); /* seek  */
  oseek=nseek;                                  /* old seek         */
  (void)fread((void*)buf,GSC_RECORD_SIZE,1,cp); /* read record */
 
  star->id     = id;
  star->alpha  = (double)((long)buf[0]*65536+(long)buf[1]*256+(long)buf[2]);
  star->alpha /= (3600.*15.*10.);
  star->delta  = (double)((long)buf[3]*65536+(long)buf[4]*256+(long)buf[5]);
  star->delta  = (double)((star->delta-(3600.*90.*10.))/(3600.*10.));
  star->mag    = (float)(buf[6]/10.);
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);

  star->spc[0] = sp0[buf[7] >> 4];                 /* upper 4 bits */
  if ((buf[7] & 0x0f) == 0x0a) star->spc[1] = ' '; /* lower 4 bits */
  else                         star->spc[1] = (buf[7] & 0x0f) + '0';
  star->spc[2] = '\0';
 
  return 1;
}
 
/* --------------------------------------------------------------------- */

int GetUSNO_SA(int zone,long id,CATstar* star)
{
  int        nseek,v,i;
  u_char     buf[16];
  float      blue,red;
  static int oseek=-1;
  static int old_zone=-1;

  if ((zone != old_zone) || (cp == NULL)) {
    char path[256];
    oseek = -1;
    if (cp) fclose(cp);
    sprintf(path, "%s/usno-sa/zone%04d.cat", catpath( ), zone*75);
    cp = fopen(path,"rb");
    if (cp == NULL) return 0;
    old_zone = zone;
  }
  nseek = (id-1)*USNO_RECORD_SIZE;              /* new seek position */
  if (nseek - oseek != USNO_RECORD_SIZE) fseek(cp,nseek,0); /* seek  */
  oseek=nseek;                                  /* old seek         */
  (void)fread((void*)buf,USNO_RECORD_SIZE,1,cp); /* read record */

  star->id = (long)zone*10000000+id;
  star->alpha  = ((double)(*(long*)(buf+0))/(3600.*15.*100.));
  star->delta  = ((double)(*(long*)(buf+4))/(3600.*100.)-90.);
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);
  v = (int)(*(long*)(buf+8));
  if (v < 0)  v = -v;
  i = v % 1000;
  if ((i > 0) && (i <= 250)) red  = (float)i/10.;
  else                       red  = 26.;
  i = (v/1000) % 1000;
  if ((i > 0) && (i <= 250)) blue = (float)i/10.;
  else                       blue = 26.;
  if ((red < 26.) && (blue < 26.)) {
    star->mag = (red+blue)/2.;
    sprintf(star->spc,"C%.1f",blue-red);
  } else {
/* Move the set of spc[0] to null from below up here to get it done
  prior to the possible return 0 which would leave it unset.  */
    star->spc[0] = '\0';
    if ((red > 25.) && (blue > 25.)) return 0;
    else if (red > 25.)              star->mag = blue;
    else                             star->mag = red;
//    star->spc[0] = '\0';
  }
#ifdef DEBUG2
  fprintf(stderr,"zone=%d, a=%f, d=%f, blue=%f, red=%f, spc=%s\n",zone,
          star->alpha,star->delta,blue,red,star->spc);
#endif
  
  return 1;
}

/* --------------------------------------------------------------------- */
 
int GetUSNO_AO(int zone,long id,CATstar* star)
{
  int        nseek;  // had.. ,v,i;
  u_char     buf[OCIW_RECORD_SIZE];
  double     blue,red,h;
  static int oseek=-1;
  static int old_zone=-1;
 
  if ((zone != old_zone) || (cp == NULL)) {
    char path[256];
    oseek = -1;
    if (cp) fclose(cp);
    sprintf(path, "%s/usno-o/zone%04d.cat", catpath( ), zone*5);
    cp = fopen(path,"rb");
    if (cp == NULL) return 0;
    old_zone = zone;
  }
  nseek = (id-1)*OCIW_RECORD_SIZE;              /* new seek position */
  if (nseek - oseek != OCIW_RECORD_SIZE) fseek(cp,nseek,0); /* seek  */
  oseek=nseek;                                  /* old seek         */
  (void)fread((void*)buf,OCIW_RECORD_SIZE,1,cp); /* read record */
 
  star->id = (unsigned long)zone*10000000+id;

  h           = (double)((long)buf[0]*65536+(long)buf[1]*256+(long)buf[2]);
  star->alpha = (h/(3600.*15.*10.));
  h           = (double)((long)buf[3]*65536+(long)buf[4]*256+(long)buf[5]);
  star->delta = (h/(3600.*10.)-90.);
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);

  red  = (double)buf[6]/10.;
  blue = (double)buf[7]/10.;
/* Set spc[0] to null here in case a return 0 is done.  */
  star->spc[0] = '\0';
  if ((red > 0.1) && (blue > 0.1)) {
    star->mag = (float)((red+blue)/2.);
    sprintf(star->spc,"C%.1f",blue-red);
  } else if ((red == 0.) && (blue == 0.)) {
    return(0);
  } else if (red > 0.1) {
    star->mag    = (float)red;
//    star->spc[0] = '\0';
  } else if (blue > 0.1) {
    star->mag    = (float)blue;
//    star->spc[0] = '\0';
  } else return(0);

#ifdef DEBUG2
  fprintf(stderr,"zone=%d, a=%f, d=%f, blue=%f, red=%f, spc=%s\n",zone,
          star->alpha,star->delta,blue,red,star->spc);
#endif
 
  return(1);
}

/* --------------------------------------------------------------------- */

int GetGSCplus(int zone,long id,CATstar* star)  /* GSCplus */
{
  int        nseek;  // had.. ,v,i;
  u_char     buf[OCIW_RECORD_SIZE];
  double     blue,red,h;
  static int oseek=-1;
  static int old_zone=-1;
 
  if ((zone != old_zone) || (cp == NULL)) {
    char path[256];
    oseek = -1;
    if (cp) fclose(cp);
    sprintf(path, "%s/GSCplus/gscp%04d.cat", catpath( ), zone*25);
    cp = fopen(path,"rb");
    if (cp == NULL) return 0;
    old_zone = zone;
  }
  nseek = (id-1)*OCIW_RECORD_SIZE;              /* new seek position */
  if (nseek - oseek != OCIW_RECORD_SIZE) fseek(cp,nseek,0); /* seek  */
  oseek=nseek;                                  /* old seek         */
  (void)fread((void*)buf,OCIW_RECORD_SIZE,1,cp); /* read record */
 
  star->id = (long)zone*10000000+id;
 
  h           = (double)((long)buf[0]*65536+(long)buf[1]*256+(long)buf[2]);
  star->alpha = (h/(3600.*15.*10.));
  h           = (double)((long)buf[3]*65536+(long)buf[4]*256+(long)buf[5]);
  star->delta = (h/(3600.*10.)-90.);
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);
 
  red  = (double)buf[6]/10.;
  blue = (double)buf[7]/10.;
/* Set spc[0] to null in case we return early. */
  star->spc[0] = '\0';
  if ((red > 0.0) && (blue > 0.0)) {
    star->mag = (float)((red+blue)/2.);
    sprintf(star->spc,"C%.1f",blue-red);
  } else if ((red == 0.) && (blue == 0.)) {
#ifdef DEBUG
    fprintf(stderr,"%f,%f: both mags==ZERO\n",star->alpha,star->delta);
#endif
    return(0);
  } else if (red > 0.0) {
    star->mag    = (float)red;
//    star->spc[0] = '\0';
  } else if (blue > 0.0) {
    star->mag    = (float)blue;
//    star->spc[0] = '\0';
  } else return(0);
 
#ifdef DEBUG2
  fprintf(stderr,"zone=%d, a=%f, d=%f, blue=%f, red=%f, spc=%s\n",zone,
          star->alpha,star->delta,blue,red,star->spc);
#endif

 
  return(1);
}

/* --------------------------------------------------------------------- */

int GetPPM(long id,CATstar *star)
{
  int         nseek;
  u_char      buf[16];
  static int  oseek=-1;                        /* old seek position       */
 
  nseek = (id-1)*PPM_RECORD_SIZE;              /* new seek position       */
  if(nseek - oseek != PPM_RECORD_SIZE) fseek(cp,nseek,0);    /* seek      */
  oseek = nseek;                               /* old seek                */
  fread(buf,PPM_RECORD_SIZE,1,cp);             /* read 12 bytes           */
 
  star->id     = (long)buf[0]*65536 + (long)buf[1]*256 + (long)buf[2];
  star->alpha  = (double)((long)buf[3]*65536+(long)buf[4]*256+(long)buf[5]);
  star->alpha /= (3600.*15.*10.);
  star->delta  = (double)((long)buf[6]*65536+(long)buf[7]*256+(long)buf[8]);
  star->delta  = (star->delta-(3600.*90.*10.))/(3600.*10.);
//  star->v = sph2vec (star->alpha*Hour, star->delta*Degree);
  star->v = had2vec (star->alpha, star->delta);
  star->mag    = ((float)buf[9]-64.)/10.;
  star->spc[0] = buf[10];
  star->spc[1] = buf[11];
  star->spc[2] = '\0';
 
  return(1);
}

/* --------------------------------------------------------------------- */

char*	cat_name (int cat_id)
/*  Return the catalog name coresponding to the cat_id integer.
  The symbolic names are defined in catalog.h  */
{
static	char	*c;
    switch (cat_id) {
	case CAT_SAO:		c = "SAO";		break;
	case CAT_PPM:		c = "PPM";		break;
	case CAT_GSC:		c = "GSC";		break;
	case CAT_GSCP:		c = "GSCP";		break;
	case CAT_USNO_SA:	c = "USNO_SA";		break;
	case CAT_USNO_AO:	c = "USNO_AO";		break;
	default:		c = "Unknown";		break;
    }
    return  c;
}

/* --------------------------------------------------------------------- */

char* cat_create_name(char *name,int cat_id,long number)
{
  /* short region,id; */
 
  switch (cat_id) {
  case CAT_GSC:
#ifdef OLD
    region = (number >> 16) & 0xffff;
    id     = (number      ) & 0xffff;
    sprintf(name,"GSC %d-%d",region,id);
#else
    sprintf(name,"GSC %ld",number);
#endif
    break;
  case CAT_SAO:
    sprintf(name,"SAO %ld",number);
    break;
  case CAT_USNO_SA:
    sprintf(name,"USNO %ld-%ld",(number/10000000)*75,number%10000000);
    break;
  case CAT_USNO_AO:
    sprintf(name,"USNO %lu-%lu",((unsigned long)number/10000000)*5,(unsigned long)number%10000000);
    break;
  case CAT_GSCP:                       /* GSCplus */
    sprintf(name,"GSC+ %ld-%ld",(number/10000000)*25,number%10000000);
    break;
  case CAT_PPM:
    sprintf(name,"PPM %ld",number);
    break;
  default:
    sprintf(name,"# %ld",number);
    break;
  }

  return name;
}

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */

