/*  cututil.c == common utility subroutines for mask cutting  */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
**                                                                   **
**                   Copyright (C) 2011 by                           **
**     Ken Clardy, Carnegie Observatories, Pasadena California.      **
**                                                                   **
**      This software is proprietary, and may be used or copied      **
**          only with the written consent of the author.             **
**              All copies must retain this notice.                  **
**                                                                   **
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* Common programs and structures for mask cutting and other machining
  subroutines are included here, so as to make a single source for these
  when developing mask cutting, and special mask cutting codes. */

/*  === Common includes === */
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	<math.h>
// #include	<strings.h>	// to satisfy lint run independently

/*  ====  GLOBAL DEFINITIONS  ====  */

const	char	CTUNAME[] = "Cut Utilities";
//const	char	CTHIVD[]  = "1";
//const	char	CTUVERS[] = ".08";

#define  RIMREP  2
// Number of times to cut rim holes, usually 1 or 2

#define  DRAWRATE  1.2
// Rate of engraving relative to "slow" rate (cut rate)

#define  INDISP  5.0
// Displacement to interior for start in terms of beam width
// Started as 2.5, now expanded

#define min(a, b)	((a) > (b) ? (b) : (a))


/*  ====  Local utility and include things  ====  */

#define  IS_CUTUTIL_C

// #include	"mgfeats.h"	// to satisfy lint run independently
// features now included by maskdat.h
#include	"cututil.h"

/*  SIMPLECURVE is defined in cututil.h */

// #include	"charplt.h"
//#include	"../utils/charplt.h"
// lint feels that the above is not needed...

/*  -- Feed rates and laser parameters  --  */
/* All feed rate and cutting parameter information is now
  found in the charplt.h header file common to all cutting. */

/*  ----  Programs for IMACS Laser machine  ----  */


int	nsides (double R, double Delt, double a)
/* Returns number of sides in a polygon plotted at radius R with
  a plotter resolution of Delt required for that polygon to be
  indistinguishable from a circle plotted at the same resoution.
  Angle a is the segment of the circle to be plotted, radians */
/* Note that Delt and R must be in same scale, and that it is only
  their ratio which is used here. */
{
int	n;
int	m;
double	f;
    if (a < 0.0) a = -a;
    if (Delt < 0.0) Delt = -Delt;
    if (R < 0.0) R = -R;
    if (Delt > 0.0) f = R / Delt;
    else  f = 900.0;	// Default n is 30.0 * a/2
    n = (int)(1.0 + 0.5 * a * sqrt (f));
    m = 2 + (int)( (a+1.0)/2.0 );
    if (n < m) n = m;
    return  n;
}


/*  ===  Curvature routines  ===  */

/* Curvature is expressed in diopters, the reciprocal of radius
  in meters.  A flat surface has zero curvature.  A radius of
  r millimeters has curvature of 0.001/r diopters.  A curvature
  of d diopters is a radius of 1000.0/d millimeters.  */

double	vhigh (double r, double c)
/* Compute vertex height in millimeters of a point at a distance
  r millimters from the vertex, with a curvature of c diopters.  */
{
double	p, q;
double	d;
double	y;
double	z, v;
    if (c == 0.0) return  0.0;	// for speed
    v = c / 2000.0;
    p = r * v;
    q = p * p;
    if (q > 0.25) return 0.0;  /* No height exists */
/* Now, if r > 333 mm, c = .809 diop, p > 0.135 and q > 0.0182  */
    if (q > 0.019) {	/* Exact solution for large vertex distance */
	y = (1.0 - sqrt (1.0 - q*4.0)) / (2.0*v);
    } else {    /* Iterative solution converges quickly */
	y = z = r * p;
	for (d=q; d > 1.2e-17; d *= q) y = z + y*y*v;
    }
    return y;
}

// Convergence of above is about 9 iterations near edge; could reduce the
// q decision to .003 to speed it a little, or keep method same over mask.
// When c is zero, v,p,q are all zero and y is also zero exactly.

// Need to change altitude to use the curvature rather than just
// the imacs mask values.  Use vhigh and a reference (flange?) height.



double	dvhi (vect2 p, vect2 c)
/*  Compute the height for position p and curvatures c, both given as
  separate x,y positions and x,y curvatures.  */
{
double	hx;
double	hy;
//double	cry;

/* First find the x height from x curvature */
    hx = vhigh (p.x, c.x);

/* Second, find the additional y height from y curvature */
    hy = vhigh (p.y, c.y/(1.0 - c.y*hx*1.e-3));
    return  hx+hy;

/* The algebraic derivation of the modified cy is:
    ry = 1000.0/c.y
    cy = 1000.0/(ry - hx)
    cy = 1000.0/(1000.0/c.y - hx)
    cy = 1.0/(1.0/c.y - hx/1000.0)
    cy = c.y/(1.0 - c.y*hx/1000.0)
  .. */
}

/* The above version of dvhi will work if c.x == c.y or if either
  c.x or c.y is exactly 0.0.  Otherwise it gives different results
  depending on whether x or y is found first.  Needs improvement.
  ... */

/*  ---  Cylinder metric -- for cutting flat mask on cylinder  ---  */

//double	cproj (double s, const double c, const double SF)
///* Projection metric for cylinder.  Given a distance s in mm along
//  the curve, and a curvature of c diopters, return the actual
//  rectangular coordinate x in mm.  Return NAN if the angular 
//  displacement of s exceeds a right angle.  Scale input s and output
//  x by scale factor SF ( x/SF = mm)  */
///* Needs #include <math.h> to work properly */
///* x is computed to full double precision (53 bit) accuracy */
//{
//double	x;	// Result
//double  A;	// Angular position
//
///* Should really use absolute value of c, but since we have
//  x = sin (s*c)/c the sign should cancel giving x ~ s with
//  the correct sign.  Usually c is positive anyway.  We don't
//  care about the sign of A and x may be either sign.  */
//
///* Radius of curvature r = 1000.0/c if c is non-zero. */
//    A = s * c / (1000.0 * SF);	// Correct for diopters to mm.
//    if (fabs(A) < 2.58096e-8) {	// That's sqrt(6 * 2^-53)
//	x = s;
//    } else if (fabs(A) >= M_PI_2) {
//	x = strtod("NAN", NULL);	// FP_NAN;
//    } else {
//	x = sin (A) * 1000.0 * SF / c;
//    }
//    return  x;
//}

//double	cunproj (double x, const double c, const double SF)
///* Inverse projection of cylinder.  Given a distance x in mm tangent
//  to the cylinder, return the actual distance along the curve of
//  curvature c diopters; in mm.  Return NAN if the input distance
//  exceeds the cylinder radius.  Scale with SF ( x/SF = mm ).
//  This is the inverse fo cproj above.  */
///* Needs #include <math.h> to work properly */
///* s is computed to full double precision (53 bit) accuracy */
//{
//double	s;	// Result
//double  A;	// Angular position
//
///* Should really use absolute value of c, but since we have
//  s = asin (x*c)/c the sign should cancel giving s ~ x with
//  the correct sign.  Usually c is positive anyway.  We don't
//  care about the sign of A and x may be either sign.  */
//
///* Radius of curvature r = 1000.0/c if c is non-zero. */
//    A = x * c / (1000.0 * SF);	// Correct for diopters to mm.
//    if (fabs(A) < 2.58096e-8) {	// That's sqrt(6 * 2^-53)
//	s = x;
//    } else if (fabs(A) >= 0.9999996829) {
//	s = strtod("NAN", NULL);	// FP_NAN;
//    } else {
//	s = asin (A) * 1000.0 * SF / c;
//    }
//    return  s;
//}


/*  ---  Common Output Formatting  ---  */

/* NOTE - We desire to move all formatting for X, Y and Z movements
  and other dimensional movements (Feed rate), and possibly some
  other utility output to subroutines so that the form of the output
  file can be made much more consistent.  */

void	Feol (FILE* f)
{
    fprintf (f, "\r\n");
}

void	Gpause (FILE* f, double s)
{
    fprintf (f, " G04 X %.2f  ( Pause )\r\n", s);
}

/* NOTE -- We may want to add a global status to output routines
  tracking whether a LF has been done since last output.  Then
  for such things as IDent, add a LF before output if none has
  happened since last output.  */
void	IDent (FILE* f)
{
    fprintf (f, "N%05d ", ++Lnum);
}

//sunifdef < #define  LDSSOFFSET
/* The symbol LDSSOFFSET is defined to implement the versions of XY
  differential and absolute output routines which will test for and
  use the (global) offset and angle parameters.  These are for support
  of the new LDSS masks which are cut with a position offset.  */
/* The define above is now STANDARD for maskgen */

// Define global storage, local to this module
static	double	sqa, cqa;	// Azimuthal angle sine, cosine
static	double	sqt, cqt;	// Tilt angle sine, cosine
static	double	qdx, qda;	// Displacement in X and Azimuthal angle
static	double	xbx, ybx;	// Absolute coordinate save
static	double	qdy;		// Displacement in Y
static	int	cf=0;		// Cylinder Flattening flag

void	setdisp (double disp, double ydisp, double angle, int cfflag) {
/* Set global things depending on requested transform */
/* Normally, both values are zero. */
// Special debug thing
#if (DEBUG_CUT > 0)
	printf (" <>Displacement %.1f @ %.1f\n", disp, angle);
#endif
    qdx = disp;
    qda = angle * Degree;

    qdy = ydisp;
    cf = cfflag;

// The global values use mm and radians (angle is in degrees)
    sincos (qda, &sqa, &cqa);
#ifdef  SIMPLECURVE
    sqt = Curve * disp * 1.0e-3;
#else
#ifdef  USE_CDAT
    sqt = CDhead[inst].curve.x * disp * 1.0e-3;
// Added factor of "disp" above 2007/9-7  KDC
#endif
#ifdef  USE_GSTO
    sqt = *(vect2*)GS_lookup(&CDhead,inst,CD_curve,0).x * disp * 1.0e-3;
#endif
#ifdef  USE_BSTO
    sqt = floatvalue(getval(&CDhead,InstCase(inst),"Curve")) * disp * 1.0e-3;
// NOTE - above is equivalent to using only x curvature, as we do elsewhere
#endif
#endif	// on SIMPLECURVE
// curve is in diopters, disp in mm, hence the 1.0e-3 scale, to
// convert disp to meters, and diopters are meters^-1.
    cqt = cos (asin (sqt));
}

/* Make sure to call setdisp (0.0, 0.0) at initial startup. */


vect2	qrot (vect2 a)
/* Apply the cqa/sqa defined rotation to vector a, return the result. */
{
vect2	result;
    result.x = a.x*cqa - a.y*sqa;
    result.y = a.x*sqa + a.y*cqa;
    return  result;
}

vect2	vdpx (vect2 p)
/* Transform vector location p into vector result applying the
  x portion of the global multi-mask transform.  Both are scaled.  */
{
double	r, z;	// For computing tilted mask case
double	A;	// Angle in cylinder case
vect2	c;	// vector curvature
vect2	result;

/* Find the curvature, and the X curvature */
#ifdef  SIMPLECURVE
    c = make2vect (Curve, 0.0);
#else
#ifdef  USE_CDAT
    c = CDhead[inst].curve;
#endif
#ifdef  USE_GSTO
    c = *(vect2*)GS_lookup(&CDhead,inst,CD_curve,0);
#endif
#ifdef  USE_BSTO
    c = vect2value(getval(&CDhead,InstCase(inst),"Curve"));
#endif
#endif  // on SIMPLECURVE

// Start with result being a null transform (the common case)
    result = p;
/* Find displaced x based on cf flag */
    if (cf) {	// The WFCCD case, cylinder flattening
//#ifdef  WFCCDsub
//	result.x = cproj (p.x, c.x, SF);
//#else
	A = p.x * c.x / (1000.0 * SF);	// Correct for diopters to mm.
// Actually, correct diopters to scaled output units being used for x.
	if (fabs(A) < 2.58096e-8) {	// That's sqrt(6 * 2^-53)
	    result.x = p.x;
	} else if (fabs(A) >= M_PI_2) {
	    result.x = strtod("NAN", NULL);	//xs = FP_NAN;
	} else {
	    result.x = sin (A) * 1000.0 * SF / c.x;
	}
//#endif
    } else {	// The LDSS case, tilted mask values (also null case)
	r = vect2norm(p) / SF;
	if (r < fabs(qdx)) {  // that is, qdx > (any point on mask)
#ifdef  SIMPLECURVE
	    z = vhigh (r, Curve);
#else
	    z = dvhi (p, c);
#endif
// The tilted (LDSS) mask computation
	    result.x = p.x * cqt + (qdx - z*sqt)*SF;
//	} else {  // null transform case
//	    result.x = p.x;
// actually, result already is set to p...
	}  // conditional on tilted LDSS mask case
    }  // conditional on cf flag
    return  result;
}  // End of vdpx


vect2	MMloc (vect2 mv)	// MultiMask scaled location function
/* Given the scaled location mv in a multi-mask, compute and return
  the actual scaled location for output.
  This utility is used for the full transforms by XYdp for differential
  positions, XYout for absolute outputs, and maskz to get the correct
  Z values for the expected output.  Does the X displacements using
  vdpx above, and the horizontal translation and rotation specified
  by qdx, qdy and qda, with qda represented by sqa and cqa.  */
{
/* This program is introduced only for WFCCD case; the old ways are
  still in use when only LDSS output is being done.
  This can (and should) be changed; it is easier to use this code
  for all output and for maskz as well.  */
vect2	ap;	// transformed vector position

    if (cf) {	// Cylinder Flattening case
	ap = (qda == 0.0) ? mv : qrot (mv);	// Sub-mask rotation
	ap.x += qdx;			// Apply sub-mask
	ap.y += qdy;			// displacements
	ap = vdpx (ap);		// Correct the X displacement to cylinder
    } else {	// No cf -- displacement of tilted mask case
	ap = (qdx == 0.0) ? mv : vdpx(mv);	// X displacement of tilted mask
	if (qda != 0.0) ap = qrot (ap);		// Global rotation
    }	// on cf flag
    return  ap;
}  // end of MMloc


void	XYdp (FILE* f, double x, double y, int e)
/* Write the transformed differential displacement x,y from last position
 saved on the file f; if (e), add end-of-line also.  */
{
/* This one has the offset computations in it... */
double	xp, yp;

vect2	a;		// original absolute location
vect2	b;		// new absolute location

    a = MMloc (make2vect (xbx, ybx));	// Original saved location
    xbx += x;		// Increment and save
    ybx += y;		// the current location
    b = MMloc (make2vect (xbx, ybx));	// New location
    xp = b.x - a.x;	// Find transformed difference
    yp = b.y - a.y;	// values for output

    if (SF > 0.1) {	// metric scale
	fprintf (f, " X%7.3f  Y%7.3f", xp, yp);
    } else {		// inch scale
	fprintf (f, " X%9.5f  Y%9.5f", xp, yp);
    }
    if (e) Feol (f);
}

void	VXYdp (FILE* f, vect2 v, int e)
{
    XYdp (f, v.x, v.y, e);
}

/* The XYdp and VXYdp are entries for differential positions to be
  output, similar to the XYout and VXYout entries.  When translation
  and rotation of the output are done, these will differ, but for
  non-transformed output they are the same.  */

void	XYout (FILE* f, double x, double y, int e)
/* Write the x, y displacements on file f after correcting for global
  multi-mask transforms needed; if (e) add end-of-line too.  */
{
/* This one has the offset computations in it... */
double	xp, yp;
vect2	a;		// absolute location
// Save the absolute position
    xbx = x;
    ybx = y;
// Find the absolute position for output
    a = MMloc (make2vect (x, y));
    xp = a.x;
    yp = a.y;

/* -- The regular xyout stuff, using xp, yp values... -- */
    if (SF > 0.1) {	// metric scale
	fprintf (f, " X%7.3f  Y%7.3f", xp, yp);
    } else {		// inch scale
	fprintf (f, " X%9.5f  Y%9.5f", xp, yp);
    }
    if (e) Feol (f);
}

void	VXYout (FILE* f, vect2 v, int e)
{
    XYout (f, v.x, v.y, e);
}

void	Zout (FILE* f, double z)
{
    if (SF > 0.1) {	// metric scale
	fprintf (f, "  Z%7.3f", z);
    } else {		// inch scale
	fprintf (f, "  Z%8.4f", z);
    }

}

void	Feedrate (FILE* f, char* s, double r)
{
    if (SF > 0.1) {	// metric scale
	fprintf (f, "%s F%5.1f", s, r);
    } else {		// inch scale
	fprintf (f, "%s F%4.1f", s, r);
    }
}



/*  ---  Laser Interface code  ---  */

void	lasermode (int mode, FILE* f)
/* Write laser mode commands to file f */
{
//static	int	gas = 0;
int	w=1;
int	p=0;

/* We collect all the laser commands into this routine, so
  they may be more easily edited... */

    switch (mode) {
	case Laser_on:
//	    if (gas) fprintf (f, " M8");
	    fprintf (f, " M04");
	    fprintf (f, " ( Laser On )");
	    Laseron = 1;
	    SetSlew = 0;
	    break;
	case Laser_off:
	    fprintf (f, " M05");	// also M09?
//	    if (gas) fprintf (f, " M9");
	    fprintf (f, " ( Laser Off )");
	    Laseron = 0;
	    SetSlew = 1;
	    break;
	case Autofocus_on:
//	    if (!(Autofocus & 2)) fprintf (f, " M16");	// autofocus on
//	    if (Autofocus & 1) fprintf (f, " M19");	// hold off
/* Since the conditionals may be confused, we turn autofocus on
  and turn the hold off unconditionally here.  */
	    fprintf (f, " M16 M19");	// autofocus on, hold off
	    Autofocus = 2;
	    fprintf (f, " ( Autofocus On )");
// Add a delay for the autofocus to settle before proceeding.
//	    Gpause (f, 0.3);
	    p = 25;
	    break;
	case Autofocus_off:
	    fprintf (f, " M17 M19");	// autofocus off, hold off
	    Autofocus = 0;
	    fprintf (f, " ( Autofocus Off )");
	    break;
	case Autofocus_hold:
	    fprintf (f, " M18");	// hold on
	    Autofocus |= 1;
	    fprintf (f, " ( Autofocus Hold )");
	    break;
// The release command is not used independently.
	case Autofocus_release:
	    fprintf (f, " M19");	// hold off
	    Autofocus &= 0XFFFE;
	    fprintf (f, " ( Autofocus Release )");
//	    Gpause (f, 0.3);
	    p = 25;
	    break;
	case Autofocus_restore:
	    if ((Autofocus & 2) == 0) lasermode (Autofocus_on, f);
	    if ( Autofocus & 1 ) lasermode (Autofocus_release, f);
	    w = 0;	// We don't actually output anything, other call does
	    break;
	case Laser_cutting:
//	    fprintf (f, " G80 C1 I60");
	    fprintf (f, " G80 P1");
//	    gas = 1;
	    fprintf (f, " ( Cutting )");
	    break;
	case Laser_engraving:
//	    fprintf (f, " G80 C0 I30 P20.0 R7.0");
	    fprintf (f, " G80 P3");
//	    gas = 0;
	    fprintf (f, " ( Engraving )");
	    break;
	case Laser_cutting2:
	    fprintf (f, " G80 P2");
//	    gas = 1;
	    fprintf (f, " ( Cutting-2 )");
	    break;
	case Laser_roughcut:
	    fprintf (f, " G80 P4");
//	    gas = 1;
	    fprintf (f, " ( Quick-cutting )");
	    break;
	case Exact_stop_on:
	    fprintf (f, " G61  ( Exact Stop Mode )");
	    break;
	case Exact_stop_off:
	    fprintf (f, " G64 ( Not exact stop )");
	    break;
	case Pause:
	    fprintf (f, " M15 ( Shutter closed )");    Feol (f);
	    fprintf (f, " M09 ( Gas off )");           Feol (f);
	    fprintf (f, " M0  ( Program Stop )");      Feol (f);
	    fprintf (f, " ( Press CYCLE START to continue. )");
	    break;
	default:
	    w = 0;
    }
    if (w) Feol (f);
    if (p) Gpause (f, (double)p * 0.01);
/* NOTE -- we may need to define feed rates above; 
  For scale independence, use 1.5*Inch*SF for example. */
}

int	onflange (vect2 p)
/* Return 1 if on flange, 0 if not.  Returns 1 if flat, too. */
{
double	r;
int	k;
#ifdef  SIMPLECURVE
    if (Curve == 0.0) return  1;
#else
#ifdef  USE_CDAT
//    if (CDhead[inst].curve == (vect2){0.0,0.0}) return 1;
    if (vect2norm(CDhead[inst].curve) == 0.0) return 1;
#endif
#ifdef  USE_GSTO
    if (*(vect2*)GS_lookup(&CDhead,inst,CD_curve,0) == (vect2){0.0,0.0}) return  1;
#endif
#ifdef  USE_BSTO
    if (vect2value(getval(&CDhead,InstCase(inst),"Curve")) == (vect2){0.0,0.0}) return 1;
#endif
#endif
    r = vect2norm(p) / SF;	// Location in mm
    if (r >= flange) k = 1;
    else  k = 0;
    return  k;
}

double	maskz (vect2 p)
/* Return the (scaled) height of the given (scaled) location */
/* Include the Z offset in all heights. */
{
double	r;
double	vz;
vect2	q;

#ifdef  SIMPLECURVE
    if (Curve == 0.0) return Zoff;
#else
#ifdef  USE_CDAT
//    if (CDhead[inst].curve == (vect2){0.0,0.0}) return Zoff;
// For some strange reason the above is not allowed, but it works just below?
    if (vect2norm (CDhead[inst].curve) == 0.0) return Zoff;
#endif
#ifdef  USE_GSTO
    if (*(vect2*)GS_lookup(&CDhead,inst,CD_curve,0) == (vect2){0.0,0.0}) return Zoff;
#endif
#ifdef  USE_BSTO
    if (vect2value(getval(&CDhead,InstCase(inst),"Curve")) == (vect2){0.0,0.0}) return Zoff;
#endif
#endif
/* This method includes the offset computation... */

/* Since we need q to be the transformed location, we should use the
  method of XYout to do this transform rather than doing it both here
  and in XYout also.  Thus a new subroutine to take mask position vector
  as input and produce transformed position vector as output.  */
    q = mul2vect (MMloc(p), 1.0/SF);	// q is in mm only, p is scaled.

    r = vect2norm (q);
    if (r > flange) {
	r = flange;
	q = make2vect (r, 0.0);	// fake flange position
    }
#ifdef  SIMPLECURVE
    vz = SF * vhigh (r, Curve);
#else
#ifdef  USE_CDAT
//    vz = SF * vhigh (r, CDhead[inst].curve);
    vz = SF * dvhi (q, CDhead[inst].curve);
#endif
#ifdef  USE_GSTO
//    vz = SF * vhigh (r, *(double*)GS_lookup(&CDhead,inst,CD_curve,0));
    vz = SF * dvhi (q, *(vect2*)GS_lookup(&CDhead,inst,CD_curve,0));
#endif
#ifdef  USE_BSTO
//    vz = SF * vhigh (r, floatvalue(getval(&CDhead,InstCase(inst),"Curve")));
    vz = SF * dvhi (q, vect2value(getval(&CDhead,InstCase(inst),"Curve")));
#endif
#endif
    return  vz + Zoff;
}

/* Avoidance routines */

static	vect2	uvpz (vect2 a, vect2  b)
/* Find unit vector perpendicular to a-b, directed to side 0,0 is on */
{
vect2	v;
    v.x = b.y - a.y;
    v.y = a.x - b.x;
    v = norm2vect (v);
// v is now vector to point p, where p-a = v
// The vector to point 0,0 is 0-a = -a
// If v dot v is same as v dot -a, we have answer, else negate.
    if (dot2vect (a,v) > 0.0) v = neg2vect (v);
    return  v;
}

#undef  DBRECURSE
//#define  DBRECURSE
/* Symbol  DBRECURSE  is used to debug recursion; if more than a small
  number of recursive calls are needed, we leave.  */

#undef  OMITAVOIDS
//#define  OMITAVOIDS
/* Symbol  OMITAVOIDS  is used to test without doing the recursion on
  the avoidances.  It should be undef for normal use.  */
  
double  newloc (vect2 a, double h, vect2 b, FILE* u)
/* Emit to file u the commands needed to move from point a to point
  b.  If h is not height at point a, move to the proper height.
  Return the height at b.  If needed, move will be split into
  several segments.  May be called recursively.  */
/* Values are scaled to current output units. */
{
#define  Recurse  -65536.0
// Detect recursive calls by h == Recurse, an exact FP value.
double	ha, hb;
double	hs;
double	ra;
//double	rb;
avoids	*px;
avoids	*dav;
vect2	ab;
vect2	c;
vect2	ac;
vect2	d;
vect2	pz;
vect2	v;
vect2	ip;
vect3	a3, b3, ab3;
vect3	v3;
double	r;
double	f;
double	rp;
double	abd;
double	abd3;
double	acd;
double	minimus;
double	s;
double	t;
double	e;
double	zz;
double	D;
int	fa, fb;
#ifdef  DBRECURSE
static	int	rk=0;		// debug recurse
#endif

//static int	kk=0;	// debug
//if (kk++ < 1) printf (" Scale = %.2f\n", SF);

// Find the actual heights at both ends
// compute ha and hb; both are zero if flat mask
    ha = maskz (a);
    hb = maskz (b);
//    if (Autofocus != 2) hb = ha;	// Flange stuff.

/* Check for a null case; if a and b are within 0.01 mm, and if
  h (not ha) and hb are within 0.01 mm, we don't need to do
  this motion, and return lastz = h */
	minimus = Laseron ? 0.001*SF : 0.01*SF;
    if (fabs(hb - h) < minimus &&
	vect2norm(sub2vect(a,b)) < minimus) {
	lastz = h;
	return  lastz;
    }

/* If we have non-recursive call, establish the feed rate here */
    if (h != Recurse) {
#ifdef  DBRECURSE
	rk = 0;		// debug recurse
#endif
/* Only establish a feed rate (slew) if the laser is off; if we are
  cutting, leave the selected feed rate as it is... */
	if (Laseron == 0 && SetSlew) Feedrate (u, " G90", quick);
// Make a z move if needed at this point to correct z.
// Omit this move if current h is over 9 mm ABOVE ha
	hs = h;
//	if (abs(h-ha)/SF > 0.001 &&	// That is .001 mm  WRONG ABS!!
	if (fabs(h-ha)/SF > 0.001 &&	// That is .001 mm
		(h-ha)/SF < 9.0 ) {
	    Zout (u, ha);
	    hs = ha;
	    if (Laseron) Feol (u);
	}
	if (Laseron == 0 && SetSlew) {
	    Feol (u);
	    SetSlew = 0;
	}
/* Above is a convoluted way of ensuring that end of line follows
  either a feed rate output or a required Z motion, and that only
  one end of line is done... (must be a better way) */
    } else {
#ifdef  DBRECURSE
	rk++;		// debug recurse
	if (rk > 10) {
	    if (rk < 15) {
		rk += 10;
		printf (" Recursive exit done.\n");	// debug
	    }
	    return h;	// debug recurse emergency exit
	}
#endif
/* In a recursive call, if we start or end on the flange, the other
  point must be set to flange height also! */
	hs = lastz;		// This was set before recurse call
	fa = onflange (a);
	fb = onflange (b);
	if (fa && !fb) hb = ha;
	if (fb && !fa) ha = hb;
    }

/* See if we have special flange logic
   An IMACS mask with either start or end on flange (and autofocus);
   If we stay on the flange, it is not special.  If we dip off
   the flange it is.  If both points are on flange and path
   goes off, just turn off autofocus.  If we go to/from flange,
   turn off autofocus and move.  Do vertical move before/after,
   and restore autofocus after.  Recursive call for the move.
   Until we code this section, replace it with an if statement
   with a conditional which does not register true.  */

    ra = 0.0;
//    if (inst == 0 && ra > flange) {	// temporarily null case
    if (inst == M_imacs && ra > flange) {	// temporarily null case
	;
    } else {

// Check for avoidances
// if we get an avoidance, find a mid point and do 2 recrusive calls
// and return result of the last call.
	ab = sub2vect (b, a);
	abd = vect2norm (ab);
#ifdef  USE_CDAT
	dav = CDhead[inst].av;
#endif
#ifdef  USE_GSTO
	dav = *(avoids**)GS_lookup(&CDhead,inst,CD_av,0);
#endif
#ifdef  USE_BSTO
	dav = getlsq (&CDhead, InstCase(inst), "Avoid");
#endif
//	for (px = cfig[inst].av; px != NULL; px = px->next) {
// STORE THE RECOVERED POINTER LOCALLY FOR THIS LOOP ACTIVITY
//	C_loop (px, cfig[inst].av) {
#ifndef  OMITAVOIDS
	C_loop (px, dav) {
/* Test to see if we are in, or go through the avoidance */
//	    c = make2vect (px->x, px->y);
	    c = mul2vect (make2vect (px->x, px->y), SF);
	    ac = sub2vect (c, a);
	    if (abd == 0.0) {
		r = f = 0.0;
	    } else {
		r = dot2vect (ac, ab) / (abd * abd);
		f = (r < 0.0) ? 0.0 : r;
		if (f > 1.0) f = 1.0;
	    }
/* r is location of nearest point on line, f is location on segment */
	    d = sum2vect (a, mul2vect(ab, f));
/* d is the nearest point in the segment; see if in zone */
	    rp = (px->r + avtool)*SF;  // Required avoidance distance
	    if (vect2norm(sub2vect(d,c)) < rp) {	// in zone
// Look for tangent from a to circle or radius rp about c
		acd = vect2norm (ac);
		s = rp / acd;	// Sin of angle between tang. and ac
		if (s < 1.0) {	// good tangent
// Find unit vector v perpendicular to ab toward 0,0
		    t = asin (s);
		    e = rp / cos (t);
		    if (e/rp > 1.5) {
			v = uvpz (a, c);
			e = 1.5 * rp;
			pz = a;
		    } else {
			v = uvpz (a, b);
			pz = c;
		    }
		} else {	// within circle, different v
		    v = uvpz (a, c);
		    e = 1.5 * rp;
		    pz = a;
		}
// Now, intermediate point is e * v away from point d (really pz)
// or point a if we are using ac normal...
		ip = sum2vect (pz, mul2vect(v, e));
// We recursively go from a to ip then b...
//		ip = mul2vect (ip, SF);	// scale back to plan
// above not needed when am=a and we are operating in plot units here...
		lastz = (h == Recurse) ? hs : h;
		zz = newloc (a,  Recurse, ip, u);
		zz = newloc (ip, Recurse,  b, u);
		return zz;
	    }	// In zone conditional
	}	// Avoidance loop
#endif  // on OMITAVOIDS

// Check for curvature splitting when autofocus is on.
// If distance exceeds autofocus range, establish a vector and
// call recursively for first segment and rest.  Return result of last.
	a3 = get3de2v (a, ha);
	b3 = get3de2v (b, hb);
	ab3 = sub3vect (b3, a3);
	abd3 = vect3norm (ab3);	// Full 3-d distance
//	D = (cfig[inst].curve == 0.0) ? 1.0e6 : 
//	    0.75 * sqrt (abs(8.0e3 * Afocus / cfig[inst].curve));
// ABOVE SHOULD BE SET AS A CONSTANT in "cutmask" program
// see cfig[inst].afoc as autofocus range!
#ifdef  USE_CDAT
	D = CDhead[inst].dmx;
#endif
#ifdef  USE_GSTO
	D = *(double*)GS_lookup (&CDhead, inst, CD_dmx, 0);
#endif
#ifdef  USE_BSTO
	D = floatvalue(getval(&CDhead, InstCase(inst), "dmx"));
#endif

	D *= SF;
// D is the maximum distance which we allow in one movement
// NOTE -- abd IS A 2-DIMENSIONAL DISTANCE; WE REALLY NEED TO
// COMPUTE USING A 3-DIMENSIONAL DISTANCE TO CLAIM SPHERICAL ACCURACY!!
	if (abd3 > D && (Autofocus & 2)) {	// Need to break it up...
// Intermediate point will be at min of D, abd3/2.0 along ab3
	    v3 = norm3vect (ab3);
	    e = abd3/2.0;
	    if (D < e) e = D;
	    ip = sum2vect (a, get2de3v (mul3vect (v3, e)));
// printf (" -- IP for curvature at %5.1f %5.1f", ip.x, ip.y);  //debug
// if (h == Recurse) printf (" R ");  printf ("\n");  //debug
// At top of recursion (h != Recurse) we turn off exact stop mode,
// if that mode is otherwise on.
	    if (h != Recurse && Astop)
		fprintf (u, " G64 ( Not exact stop )\r\n");
	    lastz = (h == Recurse) ? hs : h;
	    zz = newloc (a,  Recurse, ip, u);
	    zz = newloc (ip, Recurse,  b, u);
// At end of recursion, return exact stop mode if it should
// otherwise be on.
	    if (h != Recurse && Astop)
		fprintf (u, " G61  ( Exact Stop Mode )\r\n");
	    return zz;
	}

// Must be an OK move.  Emit commands for 3-D move unless both Z
// values are zero; in which case we do 2-D move.  Fall through to return hb.
// Also omit z move if ha and hb are identical, no z change.
	VXYout (u, b, 0);
	if ((hs != 0.0 || hb != 0.0) 
		&& ( (fabs(hs-hb)/SF) > 0.001) ) Zout (u, hb);
// We require a significant difference, not inequality here.
//		&& (hs != hb)) Zout (u, hb);  (oldway)
// Should also do that if h differs from hb unless we went to ha...
	Feol (u);
    }		// end special logic; section always done
    lastloc = b;
    lastz = hb;
    return  hb;
}		// end of newloc


/*  ---  Engraving interface program for laser  ---  */
#undef  Cdbug

void	pltimacs (vect2 v, int pen, void* u)
/* Draw vector v with given pen on unit u */
/* Pen values include: 
    -1 ==> Move to v in absolute coordinates, and keep that
	as a reference to the character start.  Other points
	will be refered to that start.
	Pen up at start of this movement.
    0 ==> Pen up; move without drawing to v
    1 ==> Pen down; draw a vector to v
*/
{
static	vect2  z = {999.0, 999.0};	// Start of character absolute
static	vect2  p;	// Previous value of v
//static	vect2  pz;	// Previous z location
static	int	f = 0;	// Previous pen condition
vect2	c;		// Current location
//double	h;		// Height
static	double	zh = 0.0;	// Absolute start height
static	double	ph;	// Previous height
double	ch;		// Current height, absolute;
double	dh;		// Relative height motion
char	com[8];		// For possible incremental command
FILE	*unit;

    unit = (FILE*) u;

#ifdef  Cdbug
printf (" -- Pen %2d to %6.2f %7.2f", pen, v.x, v.y);  //debug
#endif
/* Check first for absolute location; need to keep that
  position, and command an absolute motion... */
    if (pen < 0) {		// Absolute location; start character
//unused..	pz = z;
	z = v;		// Remember this zero location, absolute
	p = make2vect (0.0, 0.0);
/* NOTE -- Vector v, and remembered p, z are in scaled coordinates,
  and the zslew, zmask routines expect these scaled values. */

/* Do a newloc from former position done by newloc to this one,
  obtaining the absolute height at start of character. */
	zh = newloc (lastloc, lastz, v, unit);
	ph = zh;

/* Establish the incremental mode and feed rate */
//	Feedrate (unit, " G91", quick);
//	Feol (unit);
/* The above is eliminated 05/2-2.  It anticipates an incremental
  movement which may not occur (especially if character is space).
  Also, the feedrate part is redundant.  We set the incremental
  mode if we have a pen command with a previous pen command of -1. */

//	f = 0;		// Pen is left up (off)
	f = pen;	// Save the previous
#ifdef  Cdbug
printf ("\n");  //debug
#endif
	return;
    }

// During pen up/down, change feed rate appropriately
    if (pen != f) {	// Change pen position; up or down
#ifdef  Cdbug
//printf (" -- Pen %d to %.2f %.2f\n", pen, v.x, v.y);	// debug
printf (" dP %d", pen);  //debug
#endif

/* The proper setting of incremental mode is done here... */
	if (f < 0) {		// Need to set incremental
	    strcpy (com, " G91");
// Got Linux gcc message about strcpy being implicitly defined...
//	    sprintf (com, " G91");
	} else {		// Do not set incremental
	    strcpy (com, "");
//	    sprintf (com, "");
	}

	if (pen == 0) {		// Pen up
	    Feedrate (unit, com, quick);
//	    lasermode (Laser_off, unit);
	    if (f == 1) lasermode (Laser_off, unit);
	    else  Feol (unit);
	} else {		// Pen down
	    Feedrate (unit, com, slow*DRAWRATE);
	    lasermode (Laser_on, unit);
	}
	f = pen;
    }

/* Compute a displacement from last position since we are using an
  incremental mode.  Both v and p are relative to same z. */
    c = sub2vect (v, p);

/* Omit a null move.  Such a move is often given, to insure pen up at
  the end of the character.  The character generator will if pen is 1
  and any motion has been made at the end of a character, give a call
  with pen 0 and same location. */
    if (!( c.x == 0.0 && c.y == 0.0)) {
#ifdef  Cdbug
	printf (" dV %.2f %.2f", c.x, c.y);  //debug
#endif
// Add Z logic here
// Old way was:
//	VXYdp (unit, c, 1);

// New way does xy, then find if Z changes
	VXYdp (unit, c, 0);
// Find absolute position of this location, and absolute z
	lastloc = sum2vect (v, z);	// also track for newloc
	ch = maskz (lastloc);		// current absolute height
	dh = ch - ph;		// how far we "need" to move
	if (fabs(dh)/SF > 0.10) {	// Large z movement .004"
	    Zout (unit, dh);	// put in the Z movement
	    lastz = ph = ch;	// track where we are
	}
	Feol (unit);	// End the motion command line
    }

/* Remember the current position relative to the start. */
    p = v;
#ifdef  Cdbug
printf ("\n");  //debug
#endif

}

/* --- Cutting of a clearance (or other) hole --- */

double	chole (vect2 c, double d, double h, double fr, int rc, FILE* u)
/* Cut hole centered on c, of diameter d.  Output to file u.
  Tool clearance, etc. is already put in d.  Current height
  is h; return the height to which the output has set.  Repeat rc times. */
{
vect2	start;
double	t;
double	s;
double	r;
double	z;
int	k;
int	rep = rc;
char	frfld[20];

/* Limit the repeat counts */
    if (rep < 1) rep = 1;	// least repeat
    if (rep > 2) rep = 2;	// most repeat

/* Find the hole radius.  The diameter must already include allowance
  for the beam width; we cut to the exact diameter given.  */
    r = d / 2.0;

/* Position to the hole center location at quick speed */
/* Actually to start, which may be displaced from center... */
/* For cutting multi-masks, with displacement and rotation, we need
  to reference to the hole center, not the start. */
    start = c;
//    s = 3.0 * bwid;
    s = INDISP * bwid;
    t = r - s;	// displacement from center
    if (t > 0.0) {
//	start.x -= t;
	;
    } else {
	s = r;
    }
    z = newloc (lastloc, h, start, u);
/* Displace at slew rate to start location in local X coordinate */
/* Yes, this slightly increases the positioning time for holes cut
  without the angular displacement; we may fix that later.  It is
  comceptually similar to the extra slews in drawing characters. */

/* Add 06/12-13 -- turn on autofocus if it was off, and release autofocus
  if it was held; this is in support of cutting holes in special masks */
//    if (Autofocus & 2 == 0) lasermode (Autofocus_on, u);
//    if (Autofocus & 1) lasermode (Autofocus_release, u);
    lasermode (Autofocus_restore, u);

    if (t > 0.0) {
	if (SF > 0.1) {  // metric scale
	    fprintf (u, " G91 X%7.3f\r\n", -t);
	} else {	// inch scale
	    fprintf (u, " G91 X%9.5f\r\n", -t);
	}
    }

/* NOTE -- we may need to turn gas on before the M04, and wait for
  a 0.5 second pause before starting to cut; could turn on before
  doing displacement to hole edge, for example.  Then turn gas off
  at the stop cutting command. */

/* There are two ways of cutting a hole.  Usually, displace to edge of
  hole, turn laser on, draw a circle, turn laser off.  If the radius
  is very small, just turn laser on, wait a while, then turn it off. */

    if (r > 0.0) {	// Full hole drawing
	lasermode (Laser_on, u);
	if (SF > 0.1) {  // metric scale
	    fprintf (u, " G91 F%4.2f X%7.3f\r\n",	      slow, -s);
//	    fprintf (u, " G2  F%4.2f X0 Y0 I%7.3f J0\r\n",	fr,  r);
	    for (k=0; k < rep; k++) {
	    if (k == 0) sprintf (frfld, "F%4.2f", fr);
	    else        sprintf (frfld, "     ");
	    fprintf (u, " G2  %s X0 Y0 I%7.3f J0\r\n",	frfld,  r);
	    }  // end repeat loop
	    fprintf (u, " G1  F%4.2f X%7.3f\r\n",	      slow,  s);
	} else {	// inch scale
	    fprintf (u, " G91 F%4.2f X%9.5f\r\n",	      slow, -s);
//	    fprintf (u, " G2  F%4.2f X0 Y0 I%9.5f J0\r\n",	fr,  r);
	    for (k=0; k < rep; k++) {
	    if (k == 0) sprintf (frfld, "F%4.2f", fr);
	    else        sprintf (frfld, "     ");
	    fprintf (u, " G2  %s X0 Y0 I%9.5f J0\r\n",	frfld,  r);
	    }  // end repeat loop
	    fprintf (u, " G1  F%4.2f X%9.5f\r\n",	      slow,  s);
	}
	lasermode (Laser_off, u);
    } else {		// Spot burn, minimum size hole
	lasermode (Laser_on, u);
	fprintf (u, " G4 X0.5\r\n");
	lasermode (Laser_off, u);
    }
//    lasermode (Autofocus_off, u);

    return  z;
}


/*  ---  Clearance Hole  ---  */

double	pvhole (double r, double a, double d, double h, adjustment w, FILE* u)
/* Cut polar coordinate hole at radius r and angle a, to nominal
  diameter d.  Apply hole adjustments in structure w.  Commands are
  written to file u.  NOTE r,d,w in mm, scaled here.  Current height h. */
{
pvec2	ploc;
vect2	loc;
vect2	start;
double	cd;
double	cr;
double	z;
double	s;   // Small displacement for laser turn on/off
int	k;

/* Find the absolute location in x-y as a vector. */
    ploc.r = r * SF;
    ploc.t = (a + w.angle_adjust) * Degree;
    loc = rect2 (ploc);

/* Compensate for actual hole diameter */
    cd = d - w.beam_diam + w.clearance;
    cd *= SF;
    cr = cd / 2.0;

/* Call to cut the hole so described */
//    z = chole (loc, cd, h, slow, u);
/* We omit some extra feedrate output, and put in autofocus
  on and off here; so we can't use the chole stuff. */

/* Find a small distance within the hole to turn on beam.  This
  presumes the hole is much larger than the beam. */
//    s = 5.0 * bwid;
    s = INDISP * bwid;

/* Position to the hole center location at quick speed */
    start = loc;
    start.x -= cr;
    start.x += s;
    z = newloc (lastloc, h, start, u);

/* Turn on the Autofocus when we get there... */
    lasermode (Autofocus_on, u);	// Does a pause, too
/* Hold that autofocus when cutting a through hole, per autof. manual */
    lasermode (Autofocus_hold, u);

/* From chole; use same feed rate (slow) and do autofocus */
    Feedrate (u, "", slow);
    lasermode (Laser_on, u);
    if (SF > 0.1) {  // metric scale
	fprintf (u, " G91      X%7.3f\r\n",   -s);
	for (k=0; k<RIMREP; k++) {
	    if (k == 1) Feedrate (u, "", slow*2.0);
	    fprintf (u, " G2 X0 Y0 I%7.3f J0\r\n", cr);
	}
	fprintf (u, " G1       X%7.3f\r\n",    s);
    } else {        // inch scale
	fprintf (u, " G91      X%9.5f\r\n",   -s);
	for (k=0; k<RIMREP; k++) {
	    if (k == 1) Feedrate (u, "", slow*2.0);
	    fprintf (u, " G2 X0 Y0 I%9.5f J0\r\n", cr);
	}
	fprintf (u, " G1       X%9.5f\r\n",    s);
    }
    lasermode (Laser_off, u);
//    lasermode (Autofocus_hold, u);
    lasermode (Autofocus_off, u);
// Actual off, since we position extra high over flange

    return  z;
}

/*  ---  Alignment Slot  ---  */

void	a_slot (double r, double a, double d, 
		double b, adjustment w, FILE* u)
/* Cut an alignment slot around radius r and angle a, with a diameter
  (width) d, and ends displaced b from the radius location.  Apply
  hole adjustments in structure w.  Write commands to file u. */
/* NOTE r,d,b,w are in mm, scaled here */
{
pvec2	ploc, drp, dtp;
vect2	loc, dr, dt;
vect2	c, hr, ht;
vect2	hs;
double	h;
int	k;

/* How we do it:
This is similar to the clearance holes.  We specify two radii and a
width, and the angle.  (Assuming the slot is in the radial direction in
coord. system).  Then slew to one radius (say the outer) at the center,
at the given angle.  Move incrementally to the side by a given vector
v, which is the displacement from center tangentially.  Turn on the
cutting stuff, and set circular mode, then move to opposite diameter by
displacing x,y by -2v about a -v center.  Then to linear mode, move
radially by a vector -r (the radial difference turned to the angle, as
a vector), then back to circular and by x,y of 2v about center of v,
and then linear by r.  This should return us to the origin.  Turn off
the cutting stuff and done.
-- Add the ability to start (and finish) much closer to outer edge; 
instead of starting at center, start near edge and finish there.
.. */

/* Find absolute location in x-y for center. */
    ploc.r = r * SF;
    ploc.t = (a + w.angle_adjust) * Degree;
    loc = rect2 (ploc);
/* Find vectors in the radial and tangential directions */
    drp.r = b - w.beam_diam + w.clearance;
    drp.r *= SF;
    drp.t = ploc.t;
    dr = rect2 (drp);
    dtp.r = d - w.beam_diam + w.clearance;
    dtp.r *= SF;
    dtp.t = drp.t + 90.0 * Degree;
    dt = rect2 (dtp);
/* dr points out and is the length between end centers; dt points
  ccw and is the effective cutting width. */
    hr = mul2vect (dr, 0.5);
    ht = mul2vect (dt, 0.5);
/* hr and ht are half of dr, dt; for convenience */

//    hs = mul2vect (norm2vect(ht), 5.0*bwid);
    hs = mul2vect (norm2vect(ht), INDISP*bwid);
/* hs is the amount to start within the cut area... */

/* Drive to the outer center (loc + dr/2) */
    c = sum2vect (loc, hr);

/* Now find position of starting location for cut... */
    c = sum2vect (c, ht);
    c = sub2vect (c, hs);

/* Drive to safe height, and to starting location, (loc + dr/2) */
    h = maskz (c);	// should be flange height
    h = newloc (lastloc, h, c, u);

/* Turn on the Autofocus when we get there... */
    lasermode (Autofocus_on, u);	// Does a pause, too
/* Hold that autofocus when cutting a through hole, per autof. manual */
    lasermode (Autofocus_hold, u);

/* Set the feed rate once, and turn laser on */
    Feedrate (u, "", slow);

/* Turn laser on within hole */
    lasermode (Laser_on, u);

/* Duplicate this section with metric and inch formats... */
    if (SF > 0.1) {  // metric scale
/* Displace to the edge of the hole tangentially (hs) */
	fprintf (u, " G91 X%7.3f  Y%7.3f\r\n", hs.x, hs.y);
	for (k=0; k<RIMREP; k++) {	/* Cut 2 passes around */
	    if (k == 1) Feedrate (u, "", slow*2.0);
/* Start cut, do half the hole (cw, to -dt, center at -dt/2 */
	    fprintf (u, " G2  X%7.3f  Y%7.3f I%7.3f J%7.3f\r\n",
	       -dt.x, -dt.y, -ht.x, -ht.y);
/* Displace radially for slot side (-dr) */
	    fprintf (u, " G1  X%7.3f  Y%7.3f\r\n", -dr.x, -dr.y);
/* Cut circularly for other hole side (cw to dt, centered dt/2) */
	    fprintf (u, " G2  X%7.3f  Y%7.3f I%7.3f J%7.3f\r\n",
		dt.x, dt.y, ht.x, ht.y);
/* Displace radially for final side (+dr) */
	    fprintf (u, " G1  X%7.3f  Y%7.3f\r\n", dr.x, dr.y);
	}  /* End of multipass loop */
/* Back inside to finish up */
	fprintf (u, "     X%7.3f  Y%7.3f\r\n", -hs.x, -hs.y);
    } else {	// inch scale
/* Displace to the edge of the hole tangentially (dt/2) */
	fprintf (u, " G91 X%9.5f  Y%9.5f\r\n", hs.x, hs.y);
	for (k=0; k<RIMREP; k++) {	/* Cut 2 passes around */
	    if (k == 1) Feedrate (u, "", slow*2.0);
/* Start cut, do half the hole (cw, to -dt, center at -dt/2 */
	    fprintf (u, " G2  X%9.5f  Y%9.5f I%9.5f J%9.5f\r\n",
		-dt.x, -dt.y, -ht.x, -ht.y);
/* Displace radially for slot side (-dr) */
	    fprintf (u, " G1  X%9.5f  Y%9.5f\r\n", -dr.x, -dr.y);
/* Cut circularly for other hole side (cw to dt, centered dt/2) */
	    fprintf (u, " G2  X%9.5f  Y%9.5f I%9.5f J%9.5f\r\n",
		dt.x, dt.y, ht.x, ht.y);
/* Displace radially for final side (+dr) */
	    fprintf (u, " G1  X%9.5f  Y%9.5f\r\n", dr.x, dr.y);
	}  /* End of multipass loop */
/* Back inside to finish up */
	fprintf (u, "     X%9.5f  Y%9.5f\r\n", -hs.x, -hs.y);
    }

/* Stop cutting and restore modes */
    lasermode (Laser_off, u);	// Stop cutting
    lasermode (Autofocus_off, u);
// Should be positioned over 3mm high, so should be safe.
}


/* --- Cutting of a displacement, and updating... */

static	double	cutdif (vect2* last, vect2 v, double h, FILE* u)
/* Put out on unit u the differential cutting command from last
 to last+v which replaces last.  Former height is h, new height
  is returned.  */
{
double	nh;
double	dh;
// NOTE for full generality, we should introduce absolute movements,
// and use "newloc" here rather than differentials...
    *last = sum2vect (*last, v);
    nh = maskz (*last);
    dh = nh - h;

    VXYdp (u, v, 0);
    if (fabs(dh/SF) > 0.10) Zout (u, dh);
    Feol (u);

    return  nh;
}

double	cutabs (vect2* last, vect2 v, double h, FILE* u)
/* Put out on unit u the absolute cutting command from last
 to last+v which replaces last.  Former height is h, new height
  is returned.  */
{
double	nh;
// double	dh;
vect2	here;
vect2	next;
// NOTE for full generality, we should introduce absolute movements,
// and use "newloc" here rather than differentials...

    here = *last;
    next = sum2vect (here, v);

    nh = newloc (here, h, next, u);

    *last = next;

    return  nh;
}


double	cutarc (vect2* last, vect2 cd, double f, FILE* u)
/* Put out on unit u the absolute cutting command for an arc
  about center cd (relative to last) from last, ccw by f revolutions,
  and replace last with the new position, and return the height. */
{
double	nh;
vect2	here;
vect2	next;
vect2	ctr;	// Actual center
pvec2	rp;	// Polar position
double	rad;	// specified radius
double	ang;	// angular rotation
double	da;	// differential angle
double	ba;	// base angle
int	nsg;	// number of segments
int	i;	// counter

    here = *last;

// Cannot use circular mode, as we are not in a plane here; so
// we do it manually...
    ang = 2.0 * M_PI * f;
    rad = vect2norm (cd);
    nsg = 1 + nsides (rad, 0.3*bwid, ang);
// may need to move nsides from maskcut.c to above here...
    da = ang / nsg;

    nh = maskz (here);

    rp = polar2 (cd);		// negative of start radius vector
    ctr = sum2vect (here, cd);	// location of center
    ba = rp.t - M_PI;		// angle of start radius from center
    for (i=0; i<nsg; i++) {
        rp.t = ba + (i+1)*da;
	next = sum2vect (ctr, rect2 (rp));
	nh = newloc (here, nh, next, u);
	here = next;
    }

    *last = next;

    return  nh;
}



/* --- Cutting of a generalized rectangle --- */
// Parameters LongSlit and SlitGap are defined at the top of the file;
// They are globals and can be modified by a configuration file value.

#define  RCORNER
//#undef  RCORNER
/* Symbol  RCORNER  is defined to implement an experimental cut order, in
  which cutting starts at the inside point, and then instead of going
  directly to the corner goes to the x-end edge first, and then to the
  corner.  This will cut around the first corner in the same direction
  which all other corners are cut, minimizing the start corner difference
  and overlaping with the final cut.  */

double	crect (vect2 p, vect2 a, double w, double hp, double fr, int rc, FILE* u)
/* Cut a rectangle with edge at point p and centerline a; use width
  w perpendicular to a, feed rate fr, and output to file u.
  Tool diameter is in bwid (global).  Current height is hp, return
  the height to which output has been set.  Repeat rc times.  */
{
double	z;	// Height
double	x, y;	// Lengths to move
vect2	ux, uy;	// Unit vectors of rectangle
vect2	r;	// Reference location
vect2	d;	// Initial displacement vector
#ifdef  RCORNER
vect2	dx;	// Rcorner displacement
vect2	dy;	// Rcorner cross axis
#endif
double	t;	// Interior displacement length
vect2	sl, sw;	// Slit length and width vectors
vect2	next;	// Position updated
//vect2	last;	// Tracking position
double	h;
double	size;	// Size of rectangle
double	D;	// Maximum size
int	k;	// repeat index
int	rep = rc;  // repeat count, as limited

/* Compute the corner and length and width vectors, using the beam
  width; also the turn-on point.  */
    x = vect2norm (a) - bwid;
    if (x < 0.0) x = 0.0;		// Movement length
    if (rep < 1) rep = 1;	// least repeat
    if (rep > 2) rep = 2;	// most repeat

/* Test to see if the slit is "long", i.e. exceeds a pre-determined
  maximum length specified in parameter LongSlit defined above.  If it
  is, we recursively call ourselves to cut two slits, separated by
  a gap of length SlitGap defined above.  */
//    if (x > (LongSlit * SF) ) {
/*  Special feature -- do not split slits when cutting GISMO masks.  There
  should never be a long (enough) slit in a GISMO mask, and this is needed
  to properly make the GISMO aperture mask anyway.  A slightly longer
  execution for all other masks, but that is probably no problem.    */
    if (x > (LongSlit * SF) && inst != M_gismo ) {
/* The recursive slit splitting algorithm */
double	gap;
double	hx;
vect2	half;
vect2	p2;
// Find new x length of each slit part, and compute new
// slit length (just under a/2) and second slit start location.
	gap = SlitGap * SF;		// Actual length of gap
	hx = (x - gap) / 2.0;		// Length of new slit part
	half = mul2vect (a, hx/x);	// Centerline for each part
	z = crect (p, half, w, hp, fr, rep, u);	// Cut first half
	p2 = sum2vect (p, mul2vect (a, (x-hx)/x) );	// Location of part 2.
	z = crect (p2, half, w, z, fr, rep, u);	// Cut second half
	return  z;
    }
/* NOTE that the above fix for excessively long slits produces a "slit"
  which is not precisely defined by the SMDF file, in that it has gap(s)
  along its length...  */

    y = w - bwid;
    if (y < 0.0) y = 0.0;		// Movement width
    ux = norm2vect (a);
    uy = make2vect (-ux.y, ux.x);	// Rotated left 90 deg.
// Reference point is lower left corner; it is displaced from
// start p by bwid in +ux, by y/2 in -uy.
// SHOULD WE USE bwid/2 FOR MOVEMENT IN X ?? 
//    r = sum2vect (p, mul2vect (ux, bwid));
    r = sum2vect (p, mul2vect (ux, bwid*0.5));
    r = sub2vect (r, mul2vect (uy, y*0.5));
//    last = r;	// This is where we start cutting...
//    size = sqrt (x*x + y*y);	// Diagonal
    size = hypot (x, y);	// Diagonal

// Find an interior turn-on point.  If x or y is zero, we omit this.
    t = 0.5 * min (y, x);
// Also, limit movement to 2.5 times beam width.
    t = min (t, INDISP * bwid);
// Move in both ux and uy by amount t...
    d = mul2vect (sum2vect (ux, uy), t);
#ifdef  RCORNER
    dx = mul2vect (ux, t);
    dy = mul2vect (uy, t);
#endif

// Set the length and width vectors based on x,y and ux,uy...
    sl = mul2vect (ux, x);
    sw = mul2vect (uy, y);

// Locate the starting position
    next = sum2vect (r, d);

// Move to the start, and turn on the laser...
    z = newloc (lastloc, hp, next, u);
    h = z;	// For height tracking

/* Check for requirement of absolute cutting mode for large
  displacement depending on curvature */

// Find the diagonal size of rectangle (size) (done above)
// Compare to distance allowed (D)
#ifdef  USE_CDAT
    D = CDhead[inst].dmx;
#endif
#ifdef  USE_GSTO
    D = *(double*)GS_lookup (&CDhead, inst, CD_dmx, 0);
#endif
#ifdef  USE_BSTO
    D = floatvalue(getval(&CDhead, InstCase(inst), "dmx"));
#endif
    D *= SF;
    if (size > D && (Autofocus & 2)) {	// Need to break it up...
//vect2	inside;
/* This is the absolute (large) rectangle cutting mode */
// Set linear mode, absolute, and feed rate;  turn beam on
	Feedrate (u, " G90", fr);
	lasermode (Laser_on, u);

// Move to starting corner

//...

// If we have non-zero displacement, move to starting corner
//	if (t > 0.0) XYout (u, -d.x, -d.y, 1);	// FIXUP TO ABS
//	inside = next;
#ifndef  RCORNER
	h = cutabs (&next, neg2vect(d), h, u);
#else
	h = cutabs (&next, neg2vect(dx), h, u);
	h = cutabs (&next, neg2vect(dy), h, u);
#endif

// Starting in the corner, we now make the non-trivial displacements
// needed to cut the rectangle.  These are sl, sw, -sl and -sw
// ADD START LOOP FOR REPEAT COUNT CUTTING
	for (k=0; k < rep; k++) {
	next = r;
	if (x > 0.0) h = cutabs (&next, sl, h, u);


// Do the cutting for the width here
	if (y > 0.0) {
	    h = cutabs (&next, sw, h, u);
	    if (x > 0.0) {	// Cut back along length
		h = cutabs (&next, neg2vect(sl), h, u);
		h = cutabs (&next, neg2vect(sw), h, u);
	    }
	}
	}  // End repeat loop
// ADD END LOOP FOR REPEAT COUNT CUTTING

// Move back inside if we have a displacement
// These could run cutdif kind of displacement, too...
//	if (t > 0.0) VXYout (u, d, 1);		// FIXUP TO ABS
	h = cutabs (&next, d, h, u);

    } else {

/* This is the incremental (small) rectangle cutting mode */
// Set linear mode, incremental, and feed rate; also beam on
	Feedrate (u, " G91", fr);
	lasermode (Laser_on, u);

// If we have non-zero displacement, move to starting corner
// Ignore h as displacement is guaranteed under 2.5 bwid
#ifndef  RCORNER
	if (t > 0.0) XYdp (u, -d.x, -d.y, 1);
#else
	if (t > 0.0) {
	    XYdp (u, -dx.x, -dx.y, 1);
	    XYdp (u, -dy.x, -dy.y, 1);
	}
#endif

// Starting in the corner, we now make the non-trivial displacements
// needed to cut the rectangle.  These are sl, sw, -sl and -sw
// ADD START LOOP FOR REPEAT COUNT CUTTING
	for (k=0; k < rep; k++) {
	next = r;
	if (x > 0.0) h = cutdif (&next, sl, h, u);


// Do the cutting for the width here
	if (y > 0.0) {
	    h = cutdif (&next, sw, h, u);
	    if (x > 0.0) {	// Cut back along length
		h = cutdif (&next, neg2vect(sl), h, u);
		h = cutdif (&next, neg2vect(sw), h, u);
	    }
	}
	}  // end repeat loop
// ADD END LOOP FOR REPEAT COUNT CUTTING

// Move back inside if we have a displacement
// These could run cutdif kind of displacement, too...
	if (t > 0.0) VXYdp (u, d, 1);
/* End of incremental cutting mode */
    }	// End of absolute/incremental section
// Finally, turn off that laser, we are done cutting here
    lasermode (Laser_off, u);
    return  z;

// As in chole and others, we leave the incremental mode on
// when we leave, the newloc code resets it right away anyway.
}

/* add carea subroutine, incrementally cutting around a specified area,
  checking the mask radius bounds and avoidances, putting in small gaps
  at intervals, and closing the figure.  Similar to crect, with the
  addition of avoidance check.  */


	/* - - - - - - - - - - *\
	|			|
	|     Area cutting	|
	|			|
	\* - - - - - - - - - - */


static	double	limit_end (vect2 p, vect2 e, avoids* aqp, double rt,
		double rmax)
/* Find distance along p to e line from p to a point validly on the line
  given avoids and max radius.  Returns negative rmax if there is no valid
  point on the line.  */
{
vect2	c;	  	// Center
vect2	cp;
vect2	a;		// a->e (unit) vector
vect2	v;		// Desired point
vect2	q;		// Foot of normal

double	f;		// fraction
double	n;		// normal
double	h;		// q to v distance
double	d;		// distance

avoids	*px;		// Avoidance queue

// See if we are outside radius, if so, find radius point
    a = sub2vect (e, p);			// vector a->e
    f = vect2norm (a);
    if (f == 0.0) return -10.0;		// Bad p->e vector (zero length)
    a = norm2vect (a);
    if (vect2norm(p) > rmax) {
	c = make2vect (0.0, 0.0);
	cp = sub2vect (c, p);			// c relative to p
	f = nearp2 (a, cp);			// |a->q| / |a->e|
	q = sum2vect (p, mul2vect(a,f));	// foot of normal
	n = dist2 (c, q);			// normal distance
	if (n > rmax) return  -20.0;  //-rmax;		// No valid point
// point we want is along p->q, distance h from q
	h = sqrt (rmax*rmax - n*n) - 1.0e-6;	// distance from q to v
// The -1.e6 above avoids rounding errors which could cause v to appear
// to exceede rmax in tests below.
// Find point v, which is h from point q in the direction from e to a...
	v = sub2vect (q, mul2vect(a,h));
	d = dot2vect (sub2vect(v,p), a);	// our distance!
//      if (vect2norm(v) > rmax) return -23.0;	// -rmax;  //debug only
    } else {
	v = p;
	d = 0.0;
    }

//    if (vect2norm(v) > rmax) return -25.0;	// -rmax;  //debug only

// See if we are in an avoid (at v), if so, find edge of it
    C_loop (px, aqp) {
	c = mul2vect (make2vect (px->x, px->y), SF);
// continue if v is outside the avoid
	if (dist2(v,c) > (px->r + rt + 0.0)*SF) continue;
	cp = sub2vect (c, v);			// c relative to v
// See if vector from v passes through the void...
//	if (fabs(pdis2(a, cp)) > (px->r + rt + 0.0)*SF) continue;
/* If we do the above, we must also check that the point v lies near the
  avoidance center c, at least within either 2.0*px->r or within
  0.5*rmax or we don't consider the avoid at all.  This would be to test
  vect2norm(cp) or dist2(v,c)  also, as in:
	if (vect2norm(cp) > 0.5 * rmax) continue;
	if (vect2norm(cp) > 2.0 * px->r * SF) continue;
    */
// If not continued, we are in this avoid and we need to recompute v...
	f = nearp2 (a, cp);			// |v->q|
	q = sum2vect (v, mul2vect(a,f));	// foot of normal
	n = dist2 (c, q);			// normal distance
	h = (px->r + rt + 1.0e-6)*SF;		// avoidance radius
	h = sqrt (h*h - n*n);			// q to circle distance
//	v = sum2vect (q, mul2vect(a,h+1.0*SF));	// new v
	v = sum2vect (q, mul2vect(a,h));	// new v
	d = dot2vect (sub2vect(v,p), a);	// our distance!
    }

// If we have a valid point, return it
    if (vect2norm(v) > rmax) return -30.0;	// -rmax;
    return  d;
}


#define  GapSize  0.20
#define  IntSize  200.0
/*  GapSize is the size in mm of the gaps left in the cut which allow
  the chad to be carefully removed rather than falling out.
  IntSize is the interval between these gaps in mm.
  The physical gap will be smaller than the GapSize by the beam width
  which is about 0.050 mm or 0.002" best case.  Thus a GapSize under
  about 0.10 mm is not recommended.  Material thickness is usually
  0.012" or 0.304 mm.  A GapSize of 0.20 yields 0.15 mm tabs, or about
  0.006", and GapSize of 0.15 yields 0.10 mm tabs, or 0.004"    */

double	carve_edge (vect2* next, vect2 e, double z, FILE* u)
/*  Emit cutting commands onto unit u to cut a long kerf from the current
  "next" position to position e along a straight line.  Small 0.2 mm gaps
  occuring every ~100 mm and near start and end.  Laser is on at start,
  leave it on at end, use cutabs to get new positions.  Take care to
  not slew when laser is off, maintain feed rate.  */
/* Added feature:  If segment is sufficiently small, we produce no gap. */
{
vect2	vi;	// vector interval
vect2	vg;	// vector gap
vect2	vt;	// vector tail
vect2	v;	// unit vector
//vect2	p;	// point to cut to

double	w;
//double	gap = 0.125 * SF;	// should be: 0.10 * SF; use 2.5 for debug
double	gap = GapSize * SF;	// should be: 0.10 * SF; use 2.5 for debug
double	intvl = IntSize * SF;
double	tail;
double	h;	// tracking

int	n;
int	i;

// Compute the gap locations and intervals
/*
  The line contains tail, n*(gap+intvl), gap, tail for a total of
  2*tail + (n+1)*gap + n*intvl = w, the total length.
  Now, tail is fixed at the minimum of intvl/5 or w/5
  -- If total length is less than 2 x maximal tail, we let the
  number of intervals (n) become zero.
  -- If total length is less than maximal tail / 2, we let the
  number of intervals (n) become -1, and we create no gap.
  Normal values -- itvl = 200 mm, tail = 40 mm
.. */
//    w = dist2 (*next, e);
//    tail = min (intvl, w) / 5.0;
//    n = 1 + (w - 2.0*tail - gap) / (intvl+gap);
//    intvl = (w - 2.0*tail - (n+1)*gap) / n;

    w = dist2 (*next, e);
    tail = intvl / 5.0;
    if (w < 0.5 * tail) {	// Very short, no gap (< 20mm)
	tail = w;
	intvl = 0.0;
	n = -1;
    } else if (w < 2.0 * tail) {	// short, 1 gap (< 80mm)
	tail = (w - gap) / 2.0;
	intvl = 0.0;
	n = 0;
    } else {			// Long, multiple gaps
	tail = min (tail, w/5.0);
	n = 1 + (w - 2.0*tail - gap) / (intvl+gap);
	intvl = (w - 2.0*tail - (n+1)*gap) / n;
    }

// Find the vector quantities
    v = norm2vect (sub2vect(e, *next));
    vg = mul2vect (v, gap);
    vt = mul2vect (v, tail);
    vi = mul2vect (v, intvl);

// Start by cutting a tail
    h = cutabs (next, vt, z, u);

// Loop to cut gap and line n times, plus one last tail.
// This loop is not done if n < 0.
    for (i=0; i <= n; i++) {
	lasermode (Laser_off, u);
	SetSlew = 0;		// No feed rate alteration
	h = cutabs (next, vg, h, u);
//	Feedrate (u, "", fr);		// Restore rate
	lasermode (Laser_on, u);
	if (i == n) h = cutabs (next, vt, h, u);  // last is tail
	else        h = cutabs (next, vi, h, u);  // most are interval
    }

    return  h;
}


#define  SIDEGAP
//#undef  SIDEGAP
/*  Symbol  SIDEGAP  is defined to add a 0.200 mm gap midway on each
  side of the area, where the direction can change at side 2.5 and 4.5.
  This adds sitffening when cutting a relatively narrow area, and should
  not hurt when cutting a wide area.  To make sure that is true, if the
  area is wider than 40 mm this feature is omitted.
  This feature is intended for GISMO aperture mask support where extra
  gaps on the sides of the areas are needed for cutting stability.
  The GapSize and IntSize symbols defined above are used here to allow
  easy modification of these parameters in the future.    */

double	carea (vect2 p, vect2 a, double w, double hp, double fr, FILE* u, 
		avoids* aqp, double rt)
/*  Cut an area boundry like a rectangle with edge at point p and centerline
  a; use width w perpendicular to a, feed rate fr, and output file u.  Avoid
  queue aqp and tool radius r used to avoid end points.  Reduce end point
  distances as needed, and cut edges with small gaps at least every 90 mm.
  Current height is hp, return the height to which output gets set.  */
// Emulates the crect subroutine in some respects.
{
double	z;	// Height
double	x, y;	// move lengths, nominal
double	d;	// distance to real location
double	s;	// a short distance
double	h;	// tracking height
double	rmax = D_maxradi * SF;	// IMACS radius

vect2	c;	// start point of basic line
vect2	e;	// end point
vect2	t;	// target -- end point of basic line;
vect2	f;	// finish point of centerline (p is start)
vect2	ux;	// unit vector along a
vect2	uy;	// unit vector perpendicular to a
vect2	r;	// reference location
vect2	ds;	// starting displacement
vect2	next;	// position updated for cutting
#ifdef  SIDEGAP
vect2	de;	// small displacement
double	gl = 0.0;	// side gap size
#endif

/*
  Maximum radius...
  Flange radius is 313.2 set in maskcut.cfg
  Defined maxradi 309.44 in mgutils.h; we probably use that
*/

//    printf (" Area start at <%.3f %.3f>, vector <%.3f %.3f> width %.3f\n",
//	p.x, p.y,  a.x, a.y,  w);

// Find unit vectors on area
    x = vect2norm(a) - bwid;
    if (x < 0.0) x = 0.0;
    y = w - bwid;
    if (y < 0.0) y = 0.0;
    ux = norm2vect (a);
    uy = make2vect (-ux.y, ux.x);	// Rotated left pi/2
    f = sum2vect (p, a);		// end point
#ifdef  SIDEGAP
    if (w < 0.2*IntSize*SF) {
	gl = 0.5*GapSize*SF;
    } else {
	gl = 0.0;
    }
/* gl is the length of 1/2 the gap to be left on the ends of the area;
  It is zero for a sufficiently wide area.
  This is intended to add the gap to the ends of GISMO aperture mask
  areas for added stability, and to not affect larger IMACS direct 
  view areas which have actual "bent" end shapes and multiple gaps.  */
#endif

// Find first corner and correct for any edges and avoids
    c = sub2vect (p, mul2vect (uy, y*0.5));
    t = sum2vect (c, a);
    d = limit_end (c, t, aqp, rt, rmax);
    if (d < 0.0) {
	printf (" *Area corner 1 BAD %.1f\n", d);
	return hp;		// Cannot fit the area!
    }
    r = sum2vect (c, mul2vect(ux,d));	// Start here.

// Go to first corner and start
    r = sum2vect (r, mul2vect (ux, bwid*0.5));
//    r = sub2vect (r, mul2vect (uy, y*0.5));	// already done!
    s = 0.5 * min (x, y);
    s = min (s, INDISP * bwid);
    ds = mul2vect (sum2vect (ux, uy), s);
    next = sum2vect (r, ds);
// move to start and start cutting
    z = newloc (lastloc, hp, next, u);
    h = z;		// tracking?
    Feedrate (u, " G90", fr);
    lasermode (Laser_on, u);
    h = cutabs (&next, neg2vect(ds), h, u);
// Now, we are at the starting corner, currently point c (actually r)

// Find second corner and similarly correct (use subroutine)
    d = limit_end (t, c, aqp, rt, rmax);
    if (d < 0.0) { printf (" *Area corner 2 BAD %.1f\n", d); d=0.0; }
    e = sub2vect (t, mul2vect(ux,d));	// end here.
    e = sub2vect (e, mul2vect (ux, bwid*0.5));

// Draw line between the corners (1-2)
    h = carve_edge (&next, e, h, u);
    if (dodebug > 1) printf (" Completed line 1\n");

// Find corner 2.5 at far end of centerline
    d = limit_end (f, p, aqp, rt, rmax);
    if (d < 0.0) { printf (" *Area corner 2.5 BAD %.1f\n", d); d=0.0; }
    e = sub2vect (f, mul2vect(ux,d));
    e = sub2vect (e, mul2vect (ux, bwid*0.5));
#ifdef  SIDEGAP
    de = mul2vect (norm2vect(sub2vect(e,next)), gl);
/* de is a vector of length gl in direction next->e */
    e = sub2vect (e, de);
#endif  // on SIDEGAP
    h = carve_edge (&next, e, h, u);
#ifdef  SIDEGAP
    if (gl > 0.0) {
/* Create first part of gap by moving with laser off the last gl
  distance (the vector de) to the defined end point.  */
	lasermode (Laser_off, u);
	SetSlew = 0;
	h = cutabs (&next, de, h, u);
    }
#endif  // on SIDEGAP
    if (dodebug > 1) printf (" Completed line 1.5\n");

// Find third corner and draw line (2.5-3)
    c = sum2vect (c, mul2vect (uy, y));
    t = sum2vect (t, mul2vect (uy, y));
    d = limit_end (t, c, aqp, rt, rmax);
    if (d < 0.0) { printf (" *Area corner 3 BAD %.1f\n", d); d=0.0; }
    e = sub2vect (t, mul2vect(ux,d));
    e = sub2vect (e, mul2vect (ux, bwid*0.5));
#ifdef  SIDEGAP
    de = mul2vect (norm2vect(sub2vect(e,next)), gl);
/* de is a vector of length gl in direction next->e */
    if (gl > 0.0) {
/* Create the second half of the gap by moving with laser still off
  the first gl distnace (vector de) to the defined end point, and
  then turn the laser on.  Then cutting will continue to point e.  */
	SetSlew = 0;
	h = cutabs (&next, de, h, u);
	lasermode (Laser_on, u);
    }
#endif  // on SIDEGAP
    h = carve_edge (&next, e, h, u);
    if (dodebug > 1) printf (" Completed line 2\n");

// Find fourth corner and draw line (3-4)
    d = limit_end (c, t, aqp, rt, rmax);
    if (d < 0.0) { printf (" *Area corner 4 BAD %.1f\n", d); d=0.0; }
    e = sum2vect (c, mul2vect(ux,d));
    e = sum2vect (e, mul2vect (ux, bwid*0.5));
    h = carve_edge (&next, e, h, u);
    if (dodebug > 1) printf (" Completed line 3\n");

// Find 4.5 corner and draw line
    d = limit_end (p, f, aqp, rt, rmax);
    if (d < 0.0) { printf (" *Area corner 4.5 BAD %.1f\n", d); d=0.0; }
    e = sum2vect (p, mul2vect(ux,d));
    e = sum2vect (e, mul2vect (ux, bwid*0.5));
#ifdef  SIDEGAP
    de = mul2vect (norm2vect(sub2vect(e,next)), gl);
/* de is a vector of length gl in direction next->e */
    e = sub2vect (e, de);
#endif  // on SIDEGAP
    h = carve_edge (&next, e, h, u);
#ifdef  SIDEGAP
    if (gl > 0.0) {
/* Create first part of gap by moving with laser off the last gl
  distance (the vector de) to the defined end point.  */
	lasermode (Laser_off, u);
	SetSlew = 0;
	h = cutabs (&next, de, h, u);
    }
#endif  // on SIDEGAP
    if (dodebug > 1) printf (" Completed line 3.5\n");

// Draw back to first corner, then back inside. (4-1)
#ifdef  SIDEGAP
/* This is done like the gap at point 2.5 above, except we are going
  to r and  not another computed e location here.  */
    de = mul2vect (norm2vect(sub2vect(r,next)), gl);
/* de is a vector of length gl in direction next->r */
    if (gl > 0.0) {
/* Create the second half of the gap by moving with laser still off
  the first gl distnace (vector de) to the defined end point, and
  then turn the laser on.  Then cutting will continue to point r.  */
	SetSlew = 0;
	h = cutabs (&next, de, h, u);
	lasermode (Laser_on, u);
    }
#endif  // on SIDEGAP
    h = carve_edge (&next, r, h, u);
    h = cutabs (&next, ds, h, u);
    if (dodebug > 1) printf (" Completed line 4\n");

// Turn off the laser and return height.
    lasermode (Laser_off, u);

    return  z;
}

/*  ===  End Subroutines  ===  */

/*  ===  Revision History  ===  

2010/02-03 -- Version 1.15
	-- Add symbol  SIDEGAP  to allow GISMO area objects to have
	additional tabs on the sides for stability in cutting.
	-- Add symbols for gap size and tail size in carve_edge and
	area cutting.
	--> Now minimum version.

2010/01-22 -- Version 1.14
	-- Do not split slit cutting for length in GISMO mask case
	-- Add debug conditional to area cutting support messages
	--> Becomes minimum version when GISMO is released

2008/11-11 -- Version 1.13
	-- Clean up comments from cututil.c26; should change nothing.

2008/08-13 -- Version 1.12
	-- Add symbol RCORNER to start cut differently.
	--> Becomes minimum acceptable version in release 2.07

2008/02-12 -- Version 1.11
	-- Add gapless segment in carve_edge if length of interval is
	sufficiently small.
	--> This version is minimum acceptable if WFCCD is released.

2008/02-01 -- Version 1.10
	-- Add support for WFCCD and cylinder flattening mode to cut flat
	masks on the GISMO cylinder fixture.

2007/10-29 -- Version 1.09
	-- Amount to displace when starting a rectangle was 2.5 * beamwidth,
	and is now INDISP * beamwidth.  Change to 5.0 for now.
	-- Parameter INDISP used on all hole features.
	--> Minimum acceptable version at release 2.03

2007/10-03 -- Version 1.08
	-- Add repeat count for crect and chole
	-- Set as Minimum Acceptable Version for initial
	release 2.00 and correcting of error below.

2007/09-07 -- Version 1.07
	-- Fix bug causing off axis ldss3 masks to have the
	wrong tilt scale

2007/06-15 -- Version 1.06
	-- Increase carve_edge gap to 0.200 mm as a test

2006/12-13 -- Version 1.05
	-- Corrected gismo decker mask support errors

2006/11-15 -- Version 1.04
	-- Support for GISMO, including 2-d curvature

2006/10-16 -- Version 1.03
	-- Adds support of area cutting with "carea" subroutine.

2005/11-18 -- Version 1.02
	-- Modifications for new maskcut.cfg data storage.

2005/10-28 -- Version 1.01
	-- Made into separate compiled library
	-- Introduced version numbering

... */
