/* Common utility programs for .nc file plotting */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
**                                                                   **
**                  Copyright (C) 2011 by                            **
**     Ken Clardy,  Carnegie Observatories,  Pasadena California.    **
**                                                                   **
**      This software is proprietary, and may be used or copied      **
**          only with the written consent of the author.             **
**              All copies must retain this notice.                  **
**                                                                   **
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include	"ncrutil.h"

/* This file has routines used only in ncplot.  The ncrutil.c file has
  the common routines used to read a .nc file and make a linked structure
  for plotting and analysis.  These should be put directly into the
  ncplot program for clean-up.  */

/*  ====  GLOBAL DEFINITIONS  ====  */

typedef struct  {
	int     dbug;
	int	rptm;	// time feature
	int	rptz;	// z excursion
	int	inch;	// inch output feature
//	int     pwin;
	Pvfs	*cp;
	}  opts;

int	gdbug = 1;	/* Global debug flag */

int	IMACS = 0;	/* obsolete */
int	LDSS  = 0;	/* obsolete */
Mindx	MaskI  = -1;	/* Actual mask type */

vect2	curvature = {0.809061,0.809061};	// For IMACS only
double  Afocus = 1.27;		// For height offset
double  flange = 313.2;		// For IMACS flange
double	rtool = 15.0;		// For avoidance enhcnacement
double	vert = 0.00;		// vertex offset
static	int	ConfRead=0;	// reading flag


/*  ====  Set up Globals depending on Instrument  ====  */
#define  CUTFILE  "maskcut.cfg"

#ifdef  USE_CDAT
cdata	*CDhead;  // data from configuration
#endif
#ifdef  USE_GSTO
gsto	*CDhead;  // data from configuration
#endif
#ifdef  USE_BSTO
BNODE	*CDhead;  // data from configuration
#endif
// Note, above globals are used in main to kill the trees.



/*  ====  Special mask routines for IMACS and other  ====  */

double	vhigh (double r, double c)
/* Compute vertex height in millimeters of a point at a distance
  r millimters from the vertex, with a curvature of c diopters.  */
{
double	p, q;
double	d;
double	y;
double	z, v;
    if (c == 0.0) return  0.0;	// for speed
    v = c / 2000.0;
    p = r * v;
    q = p * p;
    if (q > 0.25) return 0.0;  /* No height exists */
/* Now, if r > 333 mm, c = .809 diop, p > 0.135 and q > 0.0182  */
    if (q > 0.019) {	/* Exact solution for large vertex distance */
	y = (1.0 - sqrt (1.0 - q*4.0)) / (2.0*v);
    } else {    /* Iterative solution converges quickly */
	y = z = r * p;
	for (d=q; d > 1.2e-17; d *= q) y = z + y*y*v;
    }
    return y;
}



int	get_configdata (Mindx  inst, int dl)
/* Read config file if needed, and get the configuration constants for
  the given instrument (mask) type.  Data is stored as globals.
  Return a status, 0 = OK, other = error.  Arg. dl is debug level  */
{
double	dz;
/* Get configuration data.  If desired to use a different file name,
  we can try ncplot.cfg here.  To avoid ugly message, do a findfile
  on it first, or use a debug flag. */

    if (ConfRead == 0) {
	CDhead = get_cut_data (CUTFILE, dl);
	if (CDhead == NULL) return -1;
    }
    ConfRead = 1;

#if ( DEBUG_PLT > 0 )
    if (inst < 0 || inst >= M_other) {
	printf ("  ** get_configdata instrument index = %d, set to %d.\n",
		inst, (int)M_imacs);
	inst = M_imacs;
    }
#endif

#ifdef  USE_CDAT
    flange = CDhead[inst].flange;
    rtool  = CDhead[inst].tool;
    iav    = CDhead[M_imacs].av;
    xav    = CDhead[M_lds2].av;
    if (xav == NULL) xav = CDhead[M_imacs].av;
    curvature = CDhead[inst].curve;
    Acon   = CDhead[inst].acon * 1.e-6;
    dz = CDhead[inst].dz;
#endif
// ADD ERROR MESSAGE IF NULL POINTER IS RETURNED AND SET DEFAULT VALUES
#ifdef  USE_GSTO
    flange = *(double*)GS_lookup (&CDhead, M_imacs, CD_flange, 0);
    rtool  = *(double*)GS_lookup (&CDhead, inst, CD_tool, 0);
    iav    = *(avoids**)GS_lookup (&CDhead, M_imacs, CD_av, 0);
    xav    = *(avoids**)GS_lookup (&CDhead, M_lds2, CD_av, 0);
    if (xav == NULL) xav = *(avoids**)GS_lookup (&CDhead, M_cfg, CD_av, 0);
    curvature = *(vect2*)GS_lookup (&CDhead, inst, CD_curve, 0);
/* Add acceleration constant from file */
    Acon = *(double*)GS_lookup (&CDhead, inst, CD_Acon, 0) * 1.e-6;
// Acon is defined in ncrutil.h, included by ncputil.c above
    dz = *(double)GS_lookup (&CDhead, inst, CD_dz, 0);
#endif

#ifdef  USE_BSTO
    flange = floatvalue(getval(&CDhead, "IMACS", "Flange"));
    rtool  = floatvalue(getval(&CDhead, "IMACS", "Rtool"));
    iav    = getlsq (&CDhead, "IMACS", "Avoid");
    xav    = getlsq (&CDhead, "LDSS2", "Avoid");
    if (xav == NULL) xav   = getlsq (&CDhead, "Centerfield", "Avoid");
    curvature = vect2value(getval(&CDhead, MaskCase(inst), "Curve"));
    Acon   = floatvalue(getval(&CDhead, MaskCase(inst), "Acon")) * 1.e-6;
    dz = floatvalue(getval(&CDhead, MaskCase(inst), "dz"));
/* Actually, we should get curvature and acon from current case, not imacs.*/
#endif
/* 
  Seting "curvature" above, as a global in ncputil.c is to use it in
  the maskz program specific to ncputil.c which is not the same routine
  as the maskz in cututil.c.  For addition of gismo, we need to rewrite
  the maskz program to make the curvature variable a vect2 and use it there.
 ..*/
    Afocus = dz;
//    if (inst == M_imacs) vert = vhigh (flange, curvature.x);
    if (inst == M_imacs || inst == M_lds3) vert = vhigh (flange, curvature.x);
// The vertex height applies to imacs and ldss3 mask cutting (see maskcut.c)
    else  vert = 0.0;

    return  0;
}


double	dvhi (vect2 p, vect2 c)
/*  Compute the height for position p and curvatures c, both given as
  separate x,y positions and x,y curvatures.  */
{
double	hx;
double	hy;
//double	cry;

/* First find the x height from x curvature */
    hx = vhigh (p.x, c.x);

/* Second, find the additional y height from y curvature */
    hy = vhigh (p.y, c.y/(1.0 - c.y*hx*1.e-3));
    return  hx+hy;

/* The algebraic derivation of the modified cy is:
    ry = 1000.0/c.y
    cy = 1000.0/(ry - hx)
    cy = 1000.0/(1000.0/c.y - hx)
    cy = 1.0/(1.0/c.y - hx/1000.0)
    cy = c.y/(1.0 - c.y*hx/1000.0)
  .. */
}

double	maskz (vect2 p)
/* Return the (scaled) height of the given (scaled) location */
/* Add the autofocus height for direct compare to z in file */
{
double	r;
double	vz;
//    if (!IMACS) return Afocus;	// Flat
// WE ALSO NEED TO TEST FOR GISMO CASE HERE TOO
    if (curvature.x == 0.0) return Afocus;
    r = vect2norm(p);	// Location in mm
    if (r > flange) return  Afocus;
//    vz = dvhi (p, curvature) + Afocus - vhigh (flange, curvature.x);
    vz = dvhi (p, curvature) + Afocus - vert;
    return  vz;
}

double  peakz (double z1, double z2, double z3)
/* Find peak of parabola through 3 equally spaced points;
  not to exceed the end points... */

{
double	a, b, c;
double	x;
double	z;
    c = z2;
    a = 0.5*(z1 + z3 - 2.0*c);
    b = 0.5*(z3 - z1);
    if (a == 0.0) x = 0.0;
    else x = -0.5 * b / a;
    if (x < -1.0) x = -1.0;
    if (x > 1.0) x = 1.0;
    z = c + x * (b + x*a);
    return  z;
}


static	Mindx	mask_id (mlist* a)
/* Returns a proper mask index for this list */
{
mlist	*p;
Mindx	k=-1;
/* This could be set to 0, since only very old mask files
  would have no instrument keyword.  However, we don't do
  it, and check for errors where the result is used.  */

/* Check for the instrument keyword; if found, see if it is
  one of the proper mask names, and return the index found.  */

    C_loop (p, a) {	// Loop through list
	if (p->interp == -1) {	// Instrument keyword, easy way.
// NOTE -- Should use a call to MaskNamIndx here rather than
// keeping this set of calls in the program.  This is to better
// modularize the coding...
/* Drop this section as being non-modular...
	    if      (strcasecmp(p->com, "IMACS") == 0) k = M_imacs;
	    else if (strcasecmp(p->com, "LDSS2") == 0) k = M_lds2;
	    else if (strcasecmp(p->com, "LDSS3") == 0) k = M_lds3;
	    else if (strcasecmp(p->com, "LDSS-3") == 0) k = M_lds3;
	    else if (strcasecmp(p->com, "GISMO") == 0) k = M_gismo;
	    else  k = -1;	// or maybe M_other ?
.. */
	    k = MaskNamIndx (p->com);
	    break;	// get out of loop early
	}
    }
    return  k;
}


int	isimacs2 (mlist* a)
/* Returns 1 if this is an IMACS mask, 0 if not. */
{
mlist	*p;
mlist	*q;
int	k=0;
int	v;
double	r;
vect2	loc, pre;

/* We do this by counting the number of holes near radius 323.0
  which are specified in the mask code.  A hole will have a
  circular interpolation, a final location within 0.1 of the
  previous location, and the center will be near the proper
  radius in the coordinate system.  If there are more than
  4 such holes, we recognize the IMACS mask type.  */
//    for (k=0,p=a; p != NULL; p = p->next) {	// Loop through list
    C_loop (p, a) {	// Loop through list
	if (p->interp == -1) {	// Instrument keyword, easy way.
	    k = (strcmp(p->com, "IMACS") == 0) ? 1 : 0;
	    return  k;
	}
	if (p->interp < 2 || p->interp > 3) continue;
// OK, we have a circular interpolation...
// Is it at about the right radius?
	loc = get2de3v (p->ctr);
	r = vect2norm (loc);
	if (fabs(r - 323.0) > 0.5) continue;
// OK, right radius; is it a circle?
	q = p->last;
	if (q == NULL) continue;	// don't count if no previous
	loc = get2de3v (p->v);
	pre = get2de3v (q->v);
	r = vect2norm (sub2vect(loc,pre));
// Count if r is under 0.1
	if (r < 0.1) k++;	
    }
// Now, we have IMACS if k is over 4.
//    return (k > 4);
    v = (k > 4) ? 1 : 0;
//printf (" Return %d for isimacs2.\n", v);
    return  v;
}

int	isimacs (mlist* a)
/* Returns 1 if this is an LDSS-2 mask */
/* Return 1 for IMACS mask, 2 for LDSS-2 mask, 0 for unknown */
{
mlist	*p;
int	k=0;
int	j;

/* Check for the instrument keyword; if found and the
  value is "IMACS" then return 1.  
  Otherwise, return result from isimacs2 */

//    for (k=0,p=a; p != NULL; p = p->next) {	// Loop through list
    C_loop (p, a) {	// Loop through list
	if (p->interp == -1) {	// Instrument keyword, easy way.
//printf (" Instrument = %s found in isimacs.\n", p->com);
	    k = (strcmp(p->com, "IMACS") == 0) ? 1 : 0;
//	    j = (strcmp(p->com, "LDSS:NEW") == 0) ? 2 : 0;
	    j = (strncmp(p->com, "LDSS", 4) == 0) ? 2 : 0;
//printf (" Return %d for isimacs.\n", j | k);
	    return  (j | k);
	}
    }
// Not found, return the other result.
    return  ( isimacs2 (a) );
}

int	isldss (mlist* a)
/* Returns 1 if this is an LDSS-2 mask; 0 otherwise. */
/* NOTE that this returns 0 for LDSS:NEW (or LDSS3) masks. */
{
mlist	*p;
// mlist	*q;
int	k=0;

/* Check for the instrument keyword; if found and the
  value is "LDSS2" then return 1.  Otherwise, return 0. */

//    for (k=0,p=a; p != NULL; p = p->next) {	// Loop through list
    C_loop (p, a) {	// Loop through list
	if (p->interp == -1) {	// Instrument keyword, easy way.
	    k = (strcmp(p->com, "LDSS2") == 0) ? 1 : 0;
//	    k = (strcmp(p->com, "LDSS3") == 0) ? 2 : 0;
// Non-zero for the flat mask blank case only here...
	    return  k;
	}
    }
// Not found, return 0.
    return  0;
}



Vers*	ncsource (mlist* a)
/* Returns a dynamic Vers structure containing the presumed source
  tag, and the version as found from a comment in the file.  The comment
  was saved in a special block with interp -3 by parsing program.  Here
  we retrieve that special comment and parse it further to yield the
  data for our Vers structure.  If there is no special comment, we will
  return a NULL result.  If the comment is improperly parsed, we return
  a Vers structure containing what we could find.  If the source name is
  not recognized, it is given as "Unknown".  If the version is not found
  it is returned as zero.      */
{
Vers	*v = NULL;
mlist	*p;

char	*c;
char	*d;
//char	token[32];
//int	n;
int	j;

    C_loop (p, a) {
	if (p->interp == -3) {	// source comment here
// Comment in p->com will look like this:
//  By Mask Cutting utility version 3.23.11  (or)
//  xxx...mask file by Mask Cutting utility versiion 3.24.11 (or)
//  By Some Other utilty version x.y.z

// See if we have Mask Cutting utility, and any version field...
	    c = strstr (p->com, "Mask Cutting utility ");
	    d = strstr (p->com, "version ");
// If we have no version, we continue scanning
	    if (d == NULL) continue;
// We have a version; set tag if "Mask Cutting utility" is in there,
// and possibly set another tag if another name was found...
	    v = xalloc (Vers);
	    if (c) {
		v->tag = dynamstr (MASKCTT);
	    } else {
		v->tag = dynamstr ("Unknown");
	    }
	    j = sscanf (d+8, "%f", &(v->v));
	    if (j != 1) {
		v = kill_vers (v);
	    } else  break;	// found version, end scan.

	}	// source comment conditional
    }		// loop over all lines in file
    return v;
}


/* Find extent of the window for scaling */
double	mlextent (mlist* a)
{
mlist	*p;
double	xl, xh;
double	yl, yh;
double	x, y;

/* Scan the list, finding least and greatest x and y values */
    xl = yl =  1.0e6;
    xh = yh = -1.0e6;

//    for (p=a; p != NULL; p = p->next) {
    C_loop (p, a) {
	if (p->interp < 0) continue;
	x = p->v.x;
	y = p->v.y;
// May want to also check edges of circular things?
	if (x > xh) xh = x;
	if (x < xl && x > VSET) xl = x;
	if (y > yh) yh = y;
	if (y < yl && y > VSET) yl = y;
    }

/* Now, find the maximum of both the x and y extents... */
    x = xh - xl;
    y = yh - yl;
    if (y > x) x = y;
    x *= 1.2;	// Tentative extent

/* If things are greatly assymetric, we need an offset value... */
    if (-xl > xh) xh = -xl;
    if (-yl > yh) yh = -yl;
    if (yh > xh) xh = yh;
    y = 2.0 * xh;

/* Return the greater value of our two estimates */
    if (y > x) x = y;
    return  x;
}


	/* - - - - - - - - - - - - - - *\
	|				|
	|      Timing for .nc file	|
	|				|
	\* - - - - - - - - - - - - - - */


double	report_times (mlist* head)
/* Report cutting times contained in the list */
{
// mlist	*p;
double	tot;
// int	seg = 0;
int	j;

/* The main thing is done by find_times from ncrutil.c included above */
    tot = find_times (head);


// report totals
//    printf ("  Total time = %6.3f minutes for %d segments.\n", tot, seg);

    tx[Ncond-1] = tot;
    sx[Ncond-1] = seg;
    for (j=0; j<Ncond-1; j++) dx[Ncond-1] += dx[j];

    printf ("\n   Cutting Conditions   Minutes Segments Distance\n");
    for (j=0; j<Ncond; j++) {
	printf (" %20s  %7.3f   %5d  %8.2f\n", 
		ex[j], tx[j], sx[j], dx[j]);
    }
    printf ("\n");

/* If we had read a comment at the end including the total time, check
  that it is the same as the total time computed here, and if not, we
  issue a warning message about the inconsistancy. */

    return  tot;
}

void	report_zmax (mlist* head, int imr)
/* Report max and min Z heights w.r.t. mask, while autofocus is on */
/* head is list of points, inr is non-zero for inch mode horizontal report. */
{
mlist	*p;
vect3	pv;
vect3	v;
vect2	a;
vect2	b;
vect2	c;
vect2	vlo, vhi;
double	dmax, dmin;
double	ha, hb, hc;
double	pmax;
int	paf;
int	any=0;

    dmax = -1024.0;
    dmin =  1024.0;

// Debug only -- show Afoucs and vert here
//    printf (" -- Afocus = %.2f, vert = %.2f\n", Afocus, vert);

    pv = make3vect (UNSET, UNSET, UNSET);
    vlo = vhi = make2vect (0.0, 0.0);
    paf = 0;
//    for (p=head; p!= NULL; p = p->next) {
    C_loop (p, head) {
	if (p->interp < 0) continue;  // Omit special blocks
	v = p->v;
// See if we have a valid prevous positon, or this
	if ( !( pv.x < VSET || pv.y < VSET ||
		 v.x < VSET ||  v.y < VSET ||
		pv.z < VSET ||  v.z < VSET )) {

// Consider only movement with autofocus on
	    if ((p->afoc & 2) && (paf & 2) && 
// And also with autofocus not held
		!(p->afoc & 1) ) {
//		(!(p->afoc & 1)) && (!(paf & 1)) ) {
// Find both maximum and minimum excursions from the nominal
// altitude, which we determine here for the ends and mid path
		any++;
		a = get2de3v (pv);
		b = get2de3v (v);
		c = mul2vect (sum2vect(a,b), 0.5);
		ha = pv.z - maskz (a);
		hb = v.z - maskz (b);
		hc = 0.5*(pv.z + v.z) - maskz (c);
		pmax = peakz (ha, hc, hb);
// Find extrema of ha, hb and pmax, globally
		if (ha < dmin) { dmin = ha; vlo = a; }
		if (hb < dmin) { dmin = hb; vlo = b; }
		if (pmax < dmin) { dmin = pmax; vlo = c; }
		if (ha > dmax) { dmax = ha; vhi = a; }
		if (hb > dmax) { dmax = hb; vhi = b; }
		if (pmax > dmax) { dmax = pmax; vhi = c; }
/* Note, use of c is inaccurate, for debug only */
	    }

	}
	pv = v;		// remember previous position
	paf = p->afoc;
//	if (p->next == head) break;
    }

    if (dmax < dmin) dmax = dmin = 0.0;

// report the max values found
//    printf (" With autofocus on, Z excursions were %5.2f and %5.2f from mask.",
//		dmin, dmax);
//    printf ("\n");
    if (!any) printf (" No autofocus motions.\n");
    else {
      if (imr) {	// Inch mode reporting requested
	printf (" Minimum Z of %7.2f mm at %6.2f, %6.2f inches.\n", 
		dmin, vlo.x/Inch, vlo.y/Inch);
	printf (" Maximum Z of %7.2f mm at %6.2f, %6.2f inches.\n", 
		dmax, vhi.x/Inch, vhi.y/Inch);
      } else {
	printf (" Minimum Z of %7.2f at %6.1f, %6.1f.\n", dmin, vlo.x, vlo.y);
	printf (" Maximum Z of %7.2f at %6.1f, %6.1f.\n", dmax, vhi.x, vhi.y);
      }
    }

}


/*  ====  Reading of the ".nc" File data  ====  */

#ifndef  LOCATE
/* File finding - Look for file with given name; add a .nc and
  a i.nc if needed.  Operates similar to findfile... */

static	void	findnc (char** spec, char* mode, char* name, char* dir)
{
static	char	ext[]=".nc";
char	buf[256];
char	*c;
char	*pext;

/* Look for a current result, return if so. */
    if (*spec != NULL) return;

/* Without a name, we can't do a thing. */
    if (name == NULL) return;

/* See if name has extension in it, and just try that. */
    pext = strstr (name, ext);
    if (pext) {
	findfile (spec, mode, name, dir);
    } else {
/* Otherwise, strip any trailing .xxx, append the extension and try that. */
	strcpy (buf, name);
	c = strrchr (buf, '.');
	if (c != NULL) *c = AN;
	strcat (buf, ext);
	findfile (spec, mode, buf, dir);
    }

/* See if we have it yet... */
    if (*spec != NULL) return;

/* Try the name itself with no additions */
    if (pext == NULL) findfile (spec, mode, name, dir);
    if (*spec != NULL) return;

/* Try adding i.nc after stripping extension */
    strcpy (buf, name);
    c = strrchr (buf, '.');
    if (c != NULL) *c = AN;
    strcat (buf, "i");
    strcat (buf, ext);
    findfile (spec, mode, buf, dir);
    if (*spec != NULL) return;

/* Try adding m.nc after stripping extension */
    strcpy (buf, name);
    c = strrchr (buf, '.');
    if (c != NULL) *c = AN;
    strcat (buf, "m");
    strcat (buf, ext);
    findfile (spec, mode, buf, dir);
    if (*spec != NULL) return;

/* Try adding .nc and then i.nc, m.nc to original name */
    sprintf (buf, "%s%s", name, ext);
    findfile (spec, mode, buf, dir);
    if (*spec != NULL) return;

    sprintf (buf, "%si%s", name, ext);
    findfile (spec, mode, buf, dir);
    if (*spec != NULL) return;

    sprintf (buf, "%sm%s", name, ext);
    findfile (spec, mode, buf, dir);
//    if (*spec != NULL) return;

/* Return what we have in spec if we found it... */
}
#endif  // on undef of LOCATE

#ifndef  LOCATE
/* File name searching; check names with various prefixes. */
static	void	findncf (char** spec, char* mode, char* name, char* dir)
{
char	buf[256];

/* Look for a current result, return if so. */
    if (*spec != NULL) return;

/* Without a name, we can't do a thing. */
    if (name == NULL) return;

/* Try calling findnc with various different names */
    findnc (spec, mode, name, dir);
    if (*spec != NULL) return;

    sprintf (buf, "I%s", name);
    findnc (spec, mode, buf, dir);
    if (*spec != NULL) return;

    sprintf (buf, "L%s", name);
    findnc (spec, mode, buf, dir);
    if (*spec != NULL) return;

    sprintf (buf, "G%s", name);
    findnc (spec, mode, buf, dir);
//    if (*spec != NULL) return;

}
#endif  // on undef of LOCATE


	/* - - - - - - - - - - - - - - *\
	|				|
	|      Reading of .nc file	|
	|				|
	\* - - - - - - - - - - - - - - */


/* The test file reading program... */
mlist*	Data_Processor (char* FileName, opts opt)
{
#ifdef  LOCATE
const	char	ext[]=".nc";
const	char	fmt[]="%s, I%s, L%s, G%s";
//char	buf[256];
//char	buf2[256];
char	*xn;
char	*xp;
char	*nbuf;
char	*sbuf;
#endif
FILE	*handle;
mlist	*top;
double	tused;

char	*inspec=NULL;	// Input file spec
Vers	*fv;
int	k;

//    printf (" Reads the %s file.\n", FileName);
//	fflush (stdout);


#ifdef  LOCATE
/* Make up a list of names to try */
    xn = strdup (FileName);  // in case FileName is readonly memory
    k = strlen (FileName);
    k = 2 + 4*(k+3);
    nbuf = calloc (1, 2+k*2);
    sprintf (nbuf, fmt, xn, xn, xn, xn);
/* Add to this list a similar list of names without a suffix, if the
  original name contains such a suffix.  */
    if ( (xp = strrchr(xn, '.')) ) {
	*xp = AN;	// clear the extension from our copy
	sbuf = calloc (1, k);
	sprintf (sbuf, fmt, xn, xn, xn, xn);
	strcat (nbuf, ", ");
	strcat (nbuf, sbuf);
	free (sbuf);
    }
    free (xn);

/* Make up a list of suffixes to try */
    k = 2 + 3*(strlen (ext) + 2);
    sbuf = calloc (1, k);
    sprintf (sbuf, ",%s,i%s,m%s", ext, ext, ext);
/* That above sets sufx to null, ext, i.nc, and m.nc */

/* One call now finds the file from name, suffix and directory lists */
    inspec = locfile ("r", nbuf, sbuf, ",..");
    free (nbuf);
    free (sbuf);

// Using locfile, the findncf and findnc programs are replaced.
#else  // on LOCATE
    findncf (&inspec, "r", FileName, "");
    findncf (&inspec, "r", FileName, "..");  // redundant
#endif  // on LOCATE
    if (inspec == NULL) {
	printf (" ** \"%s\" not found.\n", FileName);
	return  NULL;
    } else {
	handle = iopen (inspec, "r");
	if (handle == NULL) return NULL;
    }
    if (opt.dbug > 0) printf (" Reads the \"%s\" file.\n", inspec);
	fflush (stdout);
/* Note that inspec is a dynamic string; it must be freed. */

/* Use the Read_ncfile program included in ncrutil.c above to generate
  the actual list structure... */

    printf ("\n");
    ComTime = -1.0;
/* The "gdbug" is used only here; if > 1, it produces a very detailed
  account of the things read from the input file.  It should probably
  be equal to opt.dbug, or even opt.dbug - 1 so that opt.dbug of level 3
  would produce the detailed listing.  It is set in a statement above.
  We shall now ignore that data and set it from opt.dbug to give control
  over to the user's command line.  */
//    top = Read_ncfile (handle, opt.dbug, gdbug);
    top = Read_ncfile (handle, opt.dbug, opt.dbug - 1);

/* Check out the source version of the file here. */
    fv = ncsource (top);
    k = comp_vers (fv, opt.cp);

    if (fv == NULL) {
	printf (" ** Warning!  File made by unknown source.\n");
    } else if (k != 0 || opt.dbug > 0) {
	printf (" ** Warning!  File made by %s version %s which is ",
		fv->tag, fvstr(fv->v) );
   	switch (k) {
	    case -2: printf ("Obsolete");		break;
	    case -1: printf ("Older");			break;
	    case  0: printf ("Current");		break;
	    case  1: printf ("advanced (beta)");	break;
	    default: printf ("unknown");		break;
	}
	printf (" software.\n");
    }
/* If desired, obsolete or unknown source can be killed by killing
  all entries in top using "while (top != NULL) top = freeml(top);"
  and doing a return of NULL or top which is now NULL  */

/* Check the input file for checksum and signature */
    k = verifide (handle, dodebug-1);
// Currently, allow all errors and warnings to slide

    iclose (handle);
    if (inspec != NULL) {	// It is dynamic string...
	free (inspec);
	inspec = NULL;
    }

    MaskI = mask_id (top);
//    get_configdata (MaskI);
//    get_configdata ( (MaskI < 0) ? 0 : MaskI );  // Invalid compare
//    get_configdata ( (MaskI >= 0 && MaskI < M_other) ? MaskI : 0);
/* Use imacs case if MaskI is undefined to avoid gross errors. */
    IMACS = isimacs (top);
    if (IMACS & 1) printf ("  -- IMACS mask detected. --\n");
    if (IMACS & 2) printf ("  -- LDSS-NEW mask detected. --\n");
    if ((MaskI < 0 || MaskI >= M_other) && (IMACS & 1)) MaskI = M_imacs;
/* If we can detect IMACS mask, here in isimacs by counting flange holes as
  a last resort after the instrument keyword, then we can set the mask
  index to imacs.  Otherwise, it can remain an invalid number.  However,
  when we get the configuration data, we get the data for imacs rather than
  allowing a bad value to be passed.  This should be good since all the
  very old masks without the instrument keyword are probably imacs.  */
    get_configdata ( (MaskI >= 0 && MaskI < M_other) ? MaskI : 0, opt.dbug);
// debug only section
// printf (" MaskI = %d -- Acon = %.3g\n", MaskI, Acon);  // debug
// end debug only section
//    LDSS  = isldss (top);
    LDSS  = (MaskI == M_lds2);
    if (LDSS) printf ("  -- LDSS-2 mask detected. --\n");
    if (MaskI == M_gismo) printf ("  -- GISMO mask detected. --\n");
//#ifdef  WFCCD
    if (MaskI == M_wfcd) printf ("  -- WFCCD mask detected. --\n");
//#endif

    if (opt.dbug > 0) printf (" Mask index %d is %s\n",
	MaskI, MaskCase(MaskI) );

#if ( DEBUG_PLT > 0 )
    printf (" MaskI = %d -- Acon = %.3g\n", MaskI, Acon);  // debug
// also can error rtool, curvature, dz
    printf ("  rtool = %.3g, curvature = (%.3g, %.3g) \n",
	rtool, curvature.x, curvature.y);
#endif
    if (Acon < 0.0 || Acon > 0.001) {
	printf ("  ** Acon changed from %.3g", Acon);
	Acon = 4.5e-6;  // force resonable value
	printf (" to %.3g\n", Acon);
    }
/* If the above error message appears, rtool and curvature are also
  quite probably wrong, causing min/max Z values to be in error too.  */

    if (opt.dbug > 0) printf (" -- Acon = %.3g; Curv = %.4f; Zoff = %.3f\n", 
		Acon, curvature.x, Afocus);

/* Add the report on cutting time [optional] */
    if (opt.rptm) {
	tused = report_times (top);
    } else {
	tused = find_times (top);
    }

/* Add the report on Z excursions [optional] */
    if (opt.rptz) {
	report_zmax (top, opt.inch);
    }

/* If there was a comment of time, check the value. */
    if (ComTime > 0.0) {	// Comment exists, check the amount
	if (fabs(ComTime - tused) >= 1.0) {  // Amount is inconsistant, error message
//	    printf ("  Time from .nc file comment  is %.2f minutes.\n", ComTime);
//	    printf ("  Time from .nc file contents is %.2f minutes.\n", tused);

	    printf ("\n ** Warning - inconsistent file:\n");
	    printf (" .nc file time estimate is %.2f minutes!\n", ComTime);
	    if (opt.dbug > 0) printf (" ( Time used %.2f minutes. )\n", tused);
	}
    }

    return  top;
}
