/* SMDF to PostScript plot utility.  Reads SMDF file and plots a
  postscript file using PGPLOT routines.  The SMDF file must be one
  of the recognized subset of mask types. */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
**                                                                   **
**                   Copyright (C) 2009 by                           **
**     Ken Clardy, Carnegie Observatories, Pasadena California.      **
**                                                                   **
**      This software is proprietary, and may be used or copied      **
**          only with the written consent of the author.             **
**              All copies must retain this notice.                  **
**                                                                   **
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* The SMDF file name is the argument in the command line.  If not
  present, it is prompted for.  Product is a disk file of the same
  name, and .PS suffix containint the plot.  */

/*  ====  STANDARD INCLUDES  ====  */
#include	<stdio.h>
#include	<stdlib.h>
#include        <string.h>
#include	<unistd.h>	/* for access */
#include	<math.h>

/*  ====  GLOBAL DEFINITIONS  ====  */

const	char	PROGNAME[] = "SMDF PostScript utility";
// const	char	VERSION[]  =  "1.05";
// int     dodebug = 0;	/* Global debug flag */
// int	pwindo = 1;	/* Debug version */

typedef struct  {
	int	dbug;
	int	pwin;
	int	pnam;
	}  opts;

/*  ====  Utility programs, library programs, debug code.  ====  */

#ifdef SOLARIS
#include	"/usr/newlocal/pgplot/cpgplot.h"
#else
#ifdef  CPGPLT
#include	CPGPLT
#else
#include	<cpgplot.h>
#endif  // on CPGPLT
#endif

//#include	"mgutils.h"
#include	MGUTILH

const	char	VERSION[]  =  Xstr(SMDFPSV);

// Standard Version structure
const	Vers	Program = { NULL, NULL,  SMDFPSV, SMDFPSR, SMDFPST };
#define  APPLICATION_NAME  "SMDF PostScript utility"
//const	char	PROGNAME[] = SMDFPST;
// mgutils expect PROGNAME and VERSION strings to exist as global labels

// #include	"smplot.c"
// the smplot.c is hoplessly out of date, cannot be used...
// smplot.c is window plot of mask
/* Above was used in smdftest to do window plot */

//#include	"smdfutil.c"
/* Above routines are used to read the SMDF file format. */
/* At a later time, they may be in a library. */
/* Those routines are in mgutils library now. */

#include	"msdebug.c"	/* Debug subroutines */


/*  ====  Plotting routine(s)  ====  */

/* Plotting subroutines come first */

/* Plotting an arc */

int	nsides (double R, double Delt, double a)
/* Returns number of sides in a polygon plotted at radius R with
  a plotter resolution of Delt required for that polygon to be
  indistinguishable from a circle plotted at the same resoution.
  Angle a is the segment of the circle to be plotted. */
/* Note that Delt and R must be in same scale, and that it is only
  their ratio which is used here. */
{
int	n;
int	m;
double	f;
    if (a < 0.0) a = -a;
    if (Delt < 0.0) Delt = -Delt;
    if (R < 0.0) R = -R;
    if (Delt > 0.0) f = R / Delt;
    else  f = 900.0;	// Default n is 30.0 * a/2
    n = (int)(1.0 + 0.5 * a * sqrt (f));
    m = 2 + (int)( (a+1.0)/2.0 );
    if (n < m) n = m;
    return  n;
}

void	plot_arc (double x, double y, double r,
		double delt, double a1, double a2)
/*  Plot an arc centered on x, y with radius r from angle a1 to
  angle a2 using a polygon approximation to a circle. */
{
int	n;
int	i;
double	t;
double	a;
double	u;

    t = a2 - a1;
    n = nsides (r, delt, t);
    a = t / (double)n;
    cpgmove ( (float)(x+r*cos(a1)), (float)(y+r*sin(a1)) );
    for (i=1; i<=n; i++) {
	u = a1 + a * (double)i;
	cpgdraw ( (float)(x+r*cos(u)), (float)(y+r*sin(u)) );
    }
}


/* Subroutine to plot a circle */

/* Subroutine to plot a rectangle */

void	quadraps (vect2 q1, vect2 q2, vect2 q3, vect2 q4)
{
    cpgmove ( (float)q1.x, (float)q1.y );
    cpgdraw ( (float)q2.x, (float)q2.y );
    cpgdraw ( (float)q3.x, (float)q3.y );
    cpgdraw ( (float)q4.x, (float)q4.y );
    cpgdraw ( (float)q1.x, (float)q1.y );
}

void	rcrect (float xlo, float xhi, float ylo, float yhi, float r)
/* Plot rounded corner rectangle using world coordinates, and
  a good guess at the resolution element. */
{
double	delt;
//double	rad;
double	a1, a2;
float	x, y;

/* Make a good guess at delt... */
    delt = 0.002 * ( yhi - ylo ) / 6.0;

/* Start at bottom left and work counter-clockwise */
    x = xlo + r;
    y = ylo + r;
    cpgmove ( x, ylo );
    x = xhi - r;
    cpgdraw ( x, ylo );
    a2 = 0.0;
    a1 = -M_PI_2;
    plot_arc (x, y, r, delt, a1, a2);
    y = yhi - r;
    cpgdraw ( xhi, y );
    a1 = M_PI_2;
    plot_arc (x, y, r, delt, a2, a1);
    x = xlo + r;
    cpgdraw ( x, yhi );
    a2 = M_PI;
    plot_arc (x, y, r, delt, a1, a2);
    y = ylo + r;
    cpgdraw ( xlo, y );
    a1 = a2 + M_PI_2;
    plot_arc (x, y, r, delt, a2, a1);
}

/* Subroutine to plot mask element, slit or hole */

void	plotobj (objq*  r, float dx, float dy, float dc, int pnam)
/* Plot a mask element in world coordinates */
{

vect2	p;
vect2	p1, p2;
vect2	el, er, ew;
//vect2	q[4];
vect2	q1, q2, q3, q4;
float	xt, yt;
float	cr;

	if (r == NULL) return;

	p = r->smpos;
	ew = wslit (r->slit);
	p1 = sum2vect (p, ew);
	p2 = sub2vect (p, ew);

	if (r->slit.shape == RECTANGLE) {
	    el = lslit (r->slit);
	    er = rslit (r->slit);

	    q1 = sum2vect (p1, el);
	    q2 = sum2vect (p1, er);
	    q3 = sum2vect (p2, er);
	    q4 = sum2vect (p2, el);
	    quadraps (q1, q2, q3, q4);

//	    xt = q2.x + 1.0;
//	    yt = q2.y + 2.0;

	    xt = q2.x;
	    yt = q2.y;

/* See smplot.c for the X-window version of these plots. */
/* ALSO NEED SUBROUTINE TO DRAW ARC (PART OF CIRCLE) */

	} else if (r->slit.shape == SQUARE) {
	    er.x =  ew.y;
	    er.y = -ew.x;

	    q1 = sub2vect (p1, er);
	    q2 = sum2vect (p1, er);
	    q3 = sum2vect (p2, er);
	    q4 = sub2vect (p2, er);
	    quadraps (q1, q2, q3, q4);

//	    xt = q2.x + 1.5;
//	    yt = q2.y + 1.5;
	    xt = q2.x;
	    yt = q2.y;

	} else if (r->slit.shape == CIRCLE) {
// Find the hole radius in pixels... use X scale for this
	    cr = r->slit.width/2.0;
	    cpgcirc ( (float)p.x, (float)p.y, cr );
//	    xt = p.x + cr + 1.0;
//	    yt = p.y + cr + 1.0;
	    xt = p.x;
	    yt = p.y + cr;
	}

/* Plot the object name above, right of location. */
	if (pnam) cpgptxt ( xt+dx, yt+dy, 0.0, dc, r->dat.name );

}


void  plotcmask (Obs* obs, int pnam, float mrad)
/* Plot a circular mask data set.  Used for IMACS and
  for LDSS-3 data sets.  */
{
objq	*r;
float	hsiz, vsiz;
float	xlo, xhi;
char	buf[20];

/* Establish scaling.  The mask will be a circle, 7.8 inches in
  diameter, centered on the page.  This coresponds to mrad mm in the
  mask coordinates.  Slits are plotted within this area. */
    cpgenv ( -mrad, mrad, -mrad, mrad, 1, -2 );

/* Set up postscript for landscape mode.  MUST come after cpgenv */
    hsiz = 10.5;
    vsiz = 7.8;	// all this in inches
    xlo = (hsiz - vsiz) / 2.0;
    xhi = xlo + vsiz;
    cpgvsiz (xlo, xhi, 0.0, vsiz);

/* Plot the mask outline circle itself */
    cpgsfs (2);		// outline style
    cpgcirc (0.0, 0.0, mrad - 0.2 );	// Actual circle

/* Plot title information */
    sprintf (buf, "%.8s", obs->fname);
    cpgptxt (-0.431*mrad, 0.942*mrad, 0.0, 1.0, buf);
    sprintf (buf, "%.16s", obs->oname);
    cpgptxt ( 0.431*mrad, 0.942*mrad, 0.0, 0.0, buf);

/* Set character height for labels within graph */
    cpgsch (0.30);

/* Loop to Plot all slits/holes */
    for (r=obs->ob; r != NULL; r = r->next) {
/* Plot object, based on type... */
	plotobj (r, 1.0, 1.0, 0.0, pnam);

/* Also plots name to right; may want to differ for imacs/ldss? */
	if (r->next == obs->ob) break;
    }

}



void  plotldss  (Obs* obs, int pnam)
/* Plot a LDSS-2 style mask data set  (OBSOLETE) */
{

//float	mrad = 313.2;
float	msx = D_maxradl;	// was 88.0;
float	msy = 58.0;
objq	*r;
//int	i;
//float	hsiz, vsiz;
//float	xlo, xhi;
char	buf[20];

/* Establish scaling.  The IMACS mask will be a circle, 7.8 inches in
  diameter, centered on the page.  This coresponds to 323.2 mm in the
  mask coordinates.  Slits are plotted within this area. */
/* Establish scaling.  The LDSS-2 mask is +/- 80 mm from center in x,
  and +/- 55 mm from center in y.  Both axes must be same scale. */

    cpgenv ( -(msx+0.5), (msx+0.5), -(msy+0.5), (msy+0.5), 1, -2 );

/* Set up postscript for landscape mode.  MUST come after cpgenv */
/*  Remove all this imacs specific stuff...
    hsiz = 10.5;
    vsiz = 7.8;	// all this in inches
    xlo = (hsiz - vsiz) / 2.0;
    xhi = xlo + vsiz;
    cpgvsiz (xlo, xhi, 0.0, vsiz);
...removed */

/* Plot the mask outline circle itself */
//    cpgsfs (2);		// outline style
//    cpgcirc (0.0, 0.0, mrad - 0.2 );	// Actual circle

/* Plot the mask outline rectangle */
//    cpgmove (-msx, -msy);
//    cpgdraw ( msx, -msy);
//    cpgdraw ( msx,  msy);
//    cpgdraw (-msx,  msy);
//    cpgdraw (-msx, -msy);

/* Plot mask outline with rounded corners. */
    rcrect (-msx, msx, -msy, msy, 4.0);

/* Plot title information */
//    cpgptxt (-135.0, 295.0, 0.0, 1.0, obs->fname);
//    cpgptxt ( 135.0, 295.0, 0.0, 0.0, obs->oname);

    sprintf (buf, "%.8s", obs->fname);
    cpgptxt (-20.0, msy+5.0, 0.0, 1.0, buf);
    sprintf (buf, "%.16s", obs->oname);
    cpgptxt ( 20.0, msy+5.0, 0.0, 0.0, buf);

/* Set character height for labels within graph */
    cpgsch (0.35);

/* Loop to Plot all slits/holes */
    for (r=obs->ob; r != NULL; r = r->next) {
/* Plot object, based on type... */
	plotobj (r, -1.0, 1.0, 0.5, pnam);

/* Also plots name to right; may want to differ for imacs/ldss? */
	if (r->next == obs->ob) break;
    }

}


/* Primary plotting control program; uses PGPLOT for C stuff */

void psmask (Obs* obs, int pnam)  {
/* Plot all stuff in obs list;  postscript. */
char	outfile[24];
char	psfilenam[32];
int	i;


int	inst;

/* Find out which plot we are doing first... */

/* Try to find instrument name... */
    inst = InstIndx (obs->Instr_mt->name);

    if (inst < 0) {	/* Unknown instrument message */
	printf (" Sorry, I don't know how to plot mask data for");
	printf (" instrument \"%s\".\n", obs->Instr_mt->name);
	printf ("  ( I only know: ");
	for (i=0; i < Instruments; i++) {
	    if (i > 0) printf (", ");
	    printf ("\"%s\"", InstName(i));
	    fflush (stdout);
	}
	printf (" ).\n");
	return;
    }


/* Initialize plot, based on window/paper */

//    sprintf (outfile, "%.8s.ps\0", obs->fname);  // Changed to lower case.
    sprintf (outfile, "%.8s.ps", obs->fname);	// Changed to lower case.

    printf ("\n To plot: %.8s for %.16s;", obs->fname, obs->oname);
    printf ("\n Title: \"%s\", Inst.: \"%s\".\n",
	    obs->title, obs->Instr_mt->name);
    printf ("  Output file will be \"%s\".\n", outfile);

/* Open the pgplot file */
    strcpy (psfilenam, outfile);
    strcat (psfilenam, "/PS");
/* Mode /PS is landscape, /VPS is portrait. */

    cpgopen (psfilenam);	// Open the file for writing

/* AT THIS POINT, WE MAY NEED TO DECIDE IMACS/LDSS2 MODE, AND HAVE
  DIFFERENT WAYS OF PLOTTING... */
	switch (inst) {
	    case I_imacs:		// IMACS
//		plotcmask (obs, pnam, 313.2);
		plotcmask (obs, pnam, D_plotradi);
		break;
	    case I_lds2:		// LDSS-2
		plotldss (obs, pnam);
		break;
	    case I_lds3:		// LDSS-3
//		plotcmask (obs, pnam, 93.3);
		plotcmask (obs, pnam, D_plotradl);
		break;
	    default:
		printf (" No plotting routine for %s.\n", 
			InstName(inst));
	}

/* Terminate plot */
    cpgend ();

}

int	smdftran (char* infile, opts opt)
/* Translate smdf given by infile, return 1 if processed, 0 if not. */
{
Obs	*obs;	/* The observation structure */
int	n, m, k;	/* Debug counts */

/* If the input file argument is null, return zero */
    if (infile == NULL) return 0;
    if (*infile == '\0') return 0;

/* NOTE - null infile is different from a given infile which
  results in no input.  The latter is an error argument while
  the former is a null argument. */

/* Call to read the file */
//    printf (" -- Read smdf data from %s.\n", infile); fflush (stdout); //debug
    obs = read_smdf (infile, NULL, 0);
//    printf (" -- Data from %s was read.\n", infile); fflush (stdout); //debug


    if (obs == NULL) {
	printf (" Oops!  Did not find any observations!\n");
	printf ("  -- Cannot process this. --\n");
	return  0;
    }

/* Call dump routine to dump file */
    if (opt.dbug) dumpobs (obs);
/* Above considered debug only! */

    n = nobjects (obs);
    m = nrefholes (obs);
    k = n - m;
    printf (" %d objects; %d slits and %d reference.\n",
	n, k, m);

/* Call plotting - argument is window or paper file... */
/* Uses "pwindo" as flag; 1 to plot in window, 0 for paper. */
/* Can be set with -w flag, but debug version has it set to 1
  initially. */

/* We call plot program with window argument.  Returned is
  some output file stuff - either a name which we will need to
  change, or a pointer.  We could make the output name the
  same as input with a different extension?   That is, if input
  file is "TEST", the input routine reads "TEST.SMF" and we write
  "TEST.PS".  For this, we need to pass the input file name too. 
  Then the return would be a status.  */

/* Decide on plotting.  If window, use pwin(Obs), otherwise use
  the new psmask (obs) above */

    if (opt.pwin) {
//	pwin (obs);
	printf (" ?? Sorry, we don't do that (yet).\n");
    } else {
	psmask (obs, opt.pnam);
    }


/* For window plotting, see smdftest.c, which uses the
  include file  "smplot.c".  This does an interactive
  window plot, scalable and centerable.  */
/* Simply add -- #include  "smplot.c", and then
  do -- pwin (obs);  to get the window plotting */
/* The problem now is that the old smplot.c program uses the old
  plotting library which is obsolete; a new version using the 
  new plotting library would be needed.  See ndplot.c and ncplot.c */


/* Clean out storage of obs structure and obj queue */
/* This will be needed for real someday! */
    kill_obs (obs);
    return  1;
}



	/* - - - - - - - - *\
	|    Main Program   |
	\* - - - - - - - - */


/*  ====  MAIN PROGRAM  ====  */

#define  PROMPTLOOP

int  main (int argc, char** argv) {
char    *pp;    /* Scan variable */
char    px;     /* Scan character */
int     i;
int     dohelp = 0;
char	infile [120];	/* For prompting */
opts	opt;
namlist	*head = NULL;
char	*filename;
int	k, m, n;	/* Process counts */
namlist	*comps=NULL;


/* Get parameter as name of SMDF file to read */
/* Usual flags, debug and help */

/* Standard input logic:  If argument given, take that argument as the
  sole specification to process.  If no argument is given, prompt for
  an argument, and repeat the prompting until a null reply.  */

    opt.dbug = 0;	/* Or - put data in define above */
    opt.pwin = 0;
    opt.pnam = 0;	/* Changed default in version 0.5 */
    dodebug = 1;	/* NOTE = temporary for debugging */

/* Scan for switches and arguments */
    for (i=1; i<argc; i++) {    /* argv[0] is command name */
        pp = argv[i];
        if (*pp == '-') {       /* Scan normal switch */
            while ((px = *++pp) != '\0') {
                if (px >= 'A' && px <= 'Z') px += 'a' - 'A';
                if      (px == 'd')  dodebug = 0;  /* Debug output */
/* May want -d to cause debugging later? */
                else if (px == 'h')  dohelp = 1;  /* show help */
//		else if (px == 'n')  opt.pnam = 0;  /* Omit names */
                else if (px == 'w')  opt.pwin = 1;  /* window plot */
            }
        } else if (*pp == '+') {        /* Scan positive switch */
            while ((px = *++pp) != '\0') {
                if (px >= 'A' && px <= 'Z') px += 'a' - 'A';
                if      (px == 'd')  dodebug = 1;      /* Debug output */
		else if (px == 'n')  opt.pnam = 1;  /* Plot names */
                else if (px == 'h')  dohelp = 1;  /* show help */
            }
        } else if (*pp == '?') {
                dohelp = 1;
        } else {
#ifdef debug
            printf ("Adding %s to list.\n", pp);  /* debug */
#endif
	    head = pushtxt (pp, head);	/* Collect arguments */
//	    printf ("Argument %s.\n", pp);  /* debug */
        }
    }    

    if (dodebug && !dohelp)
	printf ("\n %s -- Version %s\n", PROGNAME, VERSION);

/* See if this version is any good, or if we continue.  */
//    comps = pushtxt (MASKCTT, comps);
    comps = pushtxt (SMDFPST, comps);
    k = check_ver (comps, NULL, NULL);
    comps = kill_name (comps);
// if feature exists, we could exit on k < -2 here.
// add pointer for Pvfs** to get version data to check the
// obs.source version for being current.

    if (dohelp) {
//	printf ("\n Slit Mask PostScript plot -- Version %s", VERSION);
	printf ("\n %s -- Version %s", PROGNAME, VERSION);
	printf ("\n  Parameters are: ");
	printf ("\n    SMDF File name(s) ");
#ifdef  PROMPTLOOP
	printf ("\n    Inputs are prompted for if not given.");
#else
	printf ("\n    Input is prompted for if not given.");
#endif
	printf ("\n   Switches");
	printf ("\n     +d = Some debug information");
	printf ("\n     -h = Display this help");
//	printf ("\n     -n = Omit plotting names");
	printf ("\n     +n = Plot names with objects");
//	printf ("\n     -p = PostScript file needed");
	printf ("\n     -w = Plot in a window");
	printf ("\n     ");
	printf ("\n   Dump(s) written to standard output.");
/*	printf ("\n   Output file name is prompted for."); */
	printf ("\n \n");
	return (1);
    }

    opt.dbug = dodebug;

/*  -- Loop to process arguments, or prompt if no arguments -- */

    for (k=n=m=0; k >= 0; ) {	// Forever loop
	if (head == NULL) {	// No arguments, or looped out
#ifdef  PROMPTLOOP
	    if (m > 0) break;	// Had real arguments, looped out
#else
	    if (n > 0) break;	// No loop after any counted argument
#endif
	    askmestr (infile, 110, "SMDF file:");
	    if (*infile == '\0') break;	// Exit on any null reply
	    filename = dynamstr (infile);
	} else {		// We have a command line argument given
	    filename = pop_name (&head);
	    if ( filename == NULL) continue;
	    if (*filename == '\0') continue;
	    m++;	// Count number of command line arguments given
	}
	n++;		// Count number of arguments requested
	k += smdftran (filename, opt);	// Count number processed
	free (filename);	// MUST be dynamic
    }

/* All done, write a message about how it came out... */
    printf ("\n Processed %d of %d files.\n", k, n);
    return  0;
}


/* Revision History

    2008/03-13 -- Version 1.07
	Upgraded use of check_ver

    2007/10-23 -- Version 1.06
	Added version control support
	Minimum Version for release 2.00 and version control.

    2007/02-21 -- Version 1.05
	Setting default radius for masks in mgutils.h
	Some clean-up of file name code

    2005/10-14 -- Version 1.00
	Attempt to modernize the PS plotting

    2004/01-09 -- Version 0.50
	Change default to no numbers
	Change output file name to .ps from .PS

    2003/07-15 -- Version 0.40
	Fix instrument name test for new names in file

    2002/12-17 -- Version 0.30
	Option to turn off the plotting of object names

    2002/3-26 -- Version 0.27
	Add round corners to LDSS-2 mask outline
	Added arc plotting and round corner rectangle routines

    2002/3-25 -- Version 0.25
	Add LDSS2 plotting - chosen as in mask cutting

    2002/3-13 -- Version 0.22
	Dump some things for debug
	Clean out un-needed comments again
	Dump of obs/objects on debug flag only

    2002/3-12 -- Version 0.20
	Standard program name global
	Multiple file name arguments possible
	Subroutine to read, process, clean each argument data
	Totally reorganized the main program
	Debug effort fixed mgutils stuff

    2001/11-21 -- Version 0.10
	Initial debug version. 

..*/

/* STANDARDIZATION OF INPUT LOGIC --
  All flag arguments are put into the "opts" structure, defined
  globally for this application.  Other arguments are put into a
  "namlist" structure, which is processed in a loop.  If none are
  found, arguments are prompted for.  This prompting may also loop
  if "PROMPTLOOP" is defined.  Any null result to a prompt will
  terminate, as will completion of command line arguments. */



